#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<CheckResultReader>::TypeImpl()
{ }

TypeImpl<CheckResultReader>::~TypeImpl()
{ }

String TypeImpl<CheckResultReader>::GetName() const
{
	return "CheckResultReader";
}

int TypeImpl<CheckResultReader>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<CheckResultReader>::GetBaseType() const
{
	return ConfigObject::TypeInstance;
}

int TypeImpl<CheckResultReader>::GetFieldId(const String& name) const
{
	int offset = ConfigObject::TypeInstance->GetFieldCount();

	switch (static_cast<int>(Utility::SDBM(name, 1))) {
		case 115:
			if (name == "spool_dir")
				return offset + 0;

			break;
	}

	return ConfigObject::TypeInstance->GetFieldId(name);
}

Field TypeImpl<CheckResultReader>::GetFieldInfo(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount();
	if (real_id < 0) { return ConfigObject::TypeInstance->GetFieldInfo(id); }
	switch (real_id) {
		case 0:
			return {0, "String", "spool_dir", "spool_dir", nullptr, 2, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<CheckResultReader>::GetFieldCount() const
{
	return 1 + ConfigObject::TypeInstance->GetFieldCount();
}

ObjectFactory TypeImpl<CheckResultReader>::GetFactory() const
{
	return TypeHelper<CheckResultReader, false>::GetFactory();
}

std::vector<String> TypeImpl<CheckResultReader>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<CheckResultReader>::GetActivationPriority() const
{
	return 100;
}

void TypeImpl<CheckResultReader>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	int real_id = fieldId - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::TypeInstance->RegisterAttributeHandler(fieldId, callback); return; }
	switch (real_id) {
		case 0:
			ObjectImpl<CheckResultReader>::OnSpoolDirChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<CheckResultReader>::Validate(int types, const ValidationUtils& utils)
{
	ConfigObject::Validate(types, utils);

	if (2 & types)
		ValidateSpoolDir(Lazy<String>([this]() { return GetSpoolDir(); }), utils);
}

void ObjectImpl<CheckResultReader>::SimpleValidateSpoolDir(const Lazy<String>& value, const ValidationUtils& utils)
{
}

ObjectImpl<CheckResultReader>::ObjectImpl()
{
	SetSpoolDir(GetDefaultSpoolDir(), true);
}

ObjectImpl<CheckResultReader>::~ObjectImpl()
{ }

void ObjectImpl<CheckResultReader>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::SetField(id, value, suppress_events, cookie); return; }
	switch (real_id) {
		case 0:
			SetSpoolDir(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<CheckResultReader>::GetField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::GetField(id); }
	switch (real_id) {
		case 0:
			return GetSpoolDir();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<CheckResultReader>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::ValidateField(id, lvalue, utils); return; }
	switch (real_id) {
		case 0:
			ValidateSpoolDir(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<CheckResultReader>::NotifyField(int id, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::NotifyField(id, cookie); return; }
	switch (real_id) {
		case 0:
			NotifySpoolDir(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<CheckResultReader>::NavigateField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::NavigateField(id); }
	throw std::runtime_error("Invalid field ID.");
}

String ObjectImpl<CheckResultReader>::GetSpoolDir() const
{
	return m_SpoolDir;
}

void ObjectImpl<CheckResultReader>::SetSpoolDir(const String& value, bool suppress_events, const Value& cookie)
{
	m_SpoolDir = value;
	if (!suppress_events)
		NotifySpoolDir(cookie);
}

void ObjectImpl<CheckResultReader>::NotifySpoolDir(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnSpoolDirChanged(static_cast<CheckResultReader *>(this), cookie);
}

String ObjectImpl<CheckResultReader>::GetDefaultSpoolDir() const
{
	 return Configuration::DataDir + "/spool/checkresults/"; 
}


boost::signals2::signal<void (const intrusive_ptr<CheckResultReader>&, const Value&)> ObjectImpl<CheckResultReader>::OnSpoolDirChanged;

void ObjectImpl<CheckResultReader>::ValidateSpoolDir(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateSpoolDir(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
