#ifndef STATUSDATAWRITER_TI
#define STATUSDATAWRITER_TI

#include "base/object.hpp"
#include "base/type.hpp"
#include "base/value.hpp"
#include "base/array.hpp"
#include "base/dictionary.hpp"
#include <boost/signals2.hpp>

#include "base/configobject.hpp"

#include "base/application.hpp"

namespace icinga
{

class StatusDataWriter;

template<>
class TypeImpl<StatusDataWriter> : public TypeImpl<ConfigObject>
{
public:
	DECLARE_PTR_TYPEDEFS(TypeImpl<StatusDataWriter>);

	TypeImpl();
	~TypeImpl() override;

	String GetName() const override;
	int GetAttributes() const override;
	Type::Ptr GetBaseType() const override;
	int GetFieldId(const String& name) const override;
	Field GetFieldInfo(int id) const override;
	int GetFieldCount() const override;
	ObjectFactory GetFactory() const override;
	std::vector<String> GetLoadDependencies() const override;
	int GetActivationPriority() const override;
public:
	void RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback) override;
};


template<>
class ObjectImpl<StatusDataWriter> : public ConfigObject
{
public:
	DECLARE_PTR_TYPEDEFS(ObjectImpl<StatusDataWriter>);

	void Validate(int types, const ValidationUtils& utils) override;
	void SimpleValidateStatusPath(const Lazy<String>& value, const ValidationUtils& utils);
	void SimpleValidateObjectsPath(const Lazy<String>& value, const ValidationUtils& utils);
	void SimpleValidateUpdateInterval(const Lazy<double>& value, const ValidationUtils& utils);
public:
	ObjectImpl<StatusDataWriter>();
public:
	~ObjectImpl<StatusDataWriter>() override;
public:
	void SetField(int id, const Value& value, bool suppress_events = false, const Value& cookie = Empty) override;
public:
	Value GetField(int id) const override;
public:
	void ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils) override;
public:
	void NotifyField(int id, const Value& cookie = Empty) override;
public:
	Object::Ptr NavigateField(int id) const override;
public:
	String GetStatusPath() const;
public:
	String GetObjectsPath() const;
public:
	double GetUpdateInterval() const;
public:
	void SetStatusPath(const String& value, bool suppress_events = false, const Value& cookie = Empty);
public:
	void SetObjectsPath(const String& value, bool suppress_events = false, const Value& cookie = Empty);
public:
	void SetUpdateInterval(double value, bool suppress_events = false, const Value& cookie = Empty);
protected:
public:
	virtual void NotifyStatusPath(const Value& cookie = Empty);
public:
	virtual void NotifyObjectsPath(const Value& cookie = Empty);
public:
	virtual void NotifyUpdateInterval(const Value& cookie = Empty);
private:
	inline String GetDefaultStatusPath() const;
private:
	inline String GetDefaultObjectsPath() const;
private:
	inline double GetDefaultUpdateInterval() const;
protected:
	virtual void ValidateStatusPath(const Lazy<String>& lvalue, const ValidationUtils& utils);
protected:
	virtual void ValidateObjectsPath(const Lazy<String>& lvalue, const ValidationUtils& utils);
protected:
	virtual void ValidateUpdateInterval(const Lazy<double>& lvalue, const ValidationUtils& utils);
private:
	String m_StatusPath;
	String m_ObjectsPath;
	double m_UpdateInterval;
public:
	static boost::signals2::signal<void (const intrusive_ptr<StatusDataWriter>&, const Value&)> OnStatusPathChanged;
	static boost::signals2::signal<void (const intrusive_ptr<StatusDataWriter>&, const Value&)> OnObjectsPathChanged;
	static boost::signals2::signal<void (const intrusive_ptr<StatusDataWriter>&, const Value&)> OnUpdateIntervalChanged;
};

}
#endif /* STATUSDATAWRITER_TI */
