#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<CheckResult>::TypeImpl()
{ }

TypeImpl<CheckResult>::~TypeImpl()
{ }

String TypeImpl<CheckResult>::GetName() const
{
	return "CheckResult";
}

int TypeImpl<CheckResult>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<CheckResult>::GetBaseType() const
{
	return Object::TypeInstance;
}

int TypeImpl<CheckResult>::GetFieldId(const String& name) const
{
	int offset = 0;

	switch (static_cast<int>(Utility::SDBM(name, 2))) {
		case 6363202:
			if (name == "active")
				return offset + 13;

			break;
		case 6494405:
			if (name == "check_source")
				return offset + 2;

			break;
		case 6494412:
			if (name == "command")
				return offset + 0;

			break;
		case 6625619:
			if (name == "exit_status")
				return offset + 7;
			if (name == "execution_start")
				return offset + 11;
			if (name == "execution_end")
				return offset + 12;

			break;
		case 7281606:
			if (name == "output")
				return offset + 1;

			break;
		case 7347189:
			if (name == "performance_data")
				return offset + 4;

			break;
		case 7543984:
			if (name == "schedule_start")
				return offset + 9;
			if (name == "schedule_end")
				return offset + 10;

			break;
		case 7544001:
			if (name == "state")
				return offset + 8;

			break;
		case 7609600:
			if (name == "ttl")
				return offset + 3;

			break;
		case 7740779:
			if (name == "vars_before")
				return offset + 5;
			if (name == "vars_after")
				return offset + 6;

			break;
	}

	return -1;
}

Field TypeImpl<CheckResult>::GetFieldInfo(int id) const
{
	switch (id) {
		case 0:
			return {0, "Value", "command", "command", nullptr, 4, 0};
		case 1:
			return {1, "String", "output", "output", nullptr, 4, 0};
		case 2:
			return {2, "String", "check_source", "check_source", nullptr, 4, 0};
		case 3:
			return {3, "Number", "ttl", "ttl", nullptr, 4, 0};
		case 4:
			return {4, "Array", "performance_data", "performance_data", nullptr, 4, 0};
		case 5:
			return {5, "Dictionary", "vars_before", "vars_before", nullptr, 4, 0};
		case 6:
			return {6, "Dictionary", "vars_after", "vars_after", nullptr, 4, 0};
		case 7:
			return {7, "Number", "exit_status", "exit_status", nullptr, 4, 0};
		case 8:
			return {8, "Number", "state", "state", nullptr, 12, 0};
		case 9:
			return {9, "Timestamp", "schedule_start", "schedule_start", nullptr, 4, 0};
		case 10:
			return {10, "Timestamp", "schedule_end", "schedule_end", nullptr, 4, 0};
		case 11:
			return {11, "Timestamp", "execution_start", "execution_start", nullptr, 4, 0};
		case 12:
			return {12, "Timestamp", "execution_end", "execution_end", nullptr, 4, 0};
		case 13:
			return {13, "Number", "active", "active", nullptr, 4, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<CheckResult>::GetFieldCount() const
{
	return 14;
}

ObjectFactory TypeImpl<CheckResult>::GetFactory() const
{
	return TypeHelper<CheckResult, false>::GetFactory();
}

std::vector<String> TypeImpl<CheckResult>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<CheckResult>::GetActivationPriority() const
{
	return 0;
}

void TypeImpl<CheckResult>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	switch (fieldId) {
		case 0:
			ObjectImpl<CheckResult>::OnCommandChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<CheckResult>::OnOutputChanged.connect(callback);
			break;
		case 2:
			ObjectImpl<CheckResult>::OnCheckSourceChanged.connect(callback);
			break;
		case 3:
			ObjectImpl<CheckResult>::OnTtlChanged.connect(callback);
			break;
		case 4:
			ObjectImpl<CheckResult>::OnPerformanceDataChanged.connect(callback);
			break;
		case 5:
			ObjectImpl<CheckResult>::OnVarsBeforeChanged.connect(callback);
			break;
		case 6:
			ObjectImpl<CheckResult>::OnVarsAfterChanged.connect(callback);
			break;
		case 7:
			ObjectImpl<CheckResult>::OnExitStatusChanged.connect(callback);
			break;
		case 8:
			ObjectImpl<CheckResult>::OnStateChanged.connect(callback);
			break;
		case 9:
			ObjectImpl<CheckResult>::OnScheduleStartChanged.connect(callback);
			break;
		case 10:
			ObjectImpl<CheckResult>::OnScheduleEndChanged.connect(callback);
			break;
		case 11:
			ObjectImpl<CheckResult>::OnExecutionStartChanged.connect(callback);
			break;
		case 12:
			ObjectImpl<CheckResult>::OnExecutionEndChanged.connect(callback);
			break;
		case 13:
			ObjectImpl<CheckResult>::OnActiveChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<CheckResult>::Validate(int types, const ValidationUtils& utils)
{
	if (4 & types)
		ValidateCommand(Lazy<Value>([this]() { return GetCommand(); }), utils);
	if (4 & types)
		ValidateOutput(Lazy<String>([this]() { return GetOutput(); }), utils);
	if (4 & types)
		ValidateCheckSource(Lazy<String>([this]() { return GetCheckSource(); }), utils);
	if (4 & types)
		ValidateTtl(Lazy<double>([this]() { return GetTtl(); }), utils);
	if (4 & types)
		ValidatePerformanceData(Lazy<Array::Ptr>([this]() { return GetPerformanceData(); }), utils);
	if (4 & types)
		ValidateVarsBefore(Lazy<Dictionary::Ptr>([this]() { return GetVarsBefore(); }), utils);
	if (4 & types)
		ValidateVarsAfter(Lazy<Dictionary::Ptr>([this]() { return GetVarsAfter(); }), utils);
	if (4 & types)
		ValidateExitStatus(Lazy<int>([this]() { return GetExitStatus(); }), utils);
	if (4 & types)
		ValidateState(Lazy<ServiceState>([this]() { return GetState(); }), utils);
	if (4 & types)
		ValidateScheduleStart(Lazy<Timestamp>([this]() { return GetScheduleStart(); }), utils);
	if (4 & types)
		ValidateScheduleEnd(Lazy<Timestamp>([this]() { return GetScheduleEnd(); }), utils);
	if (4 & types)
		ValidateExecutionStart(Lazy<Timestamp>([this]() { return GetExecutionStart(); }), utils);
	if (4 & types)
		ValidateExecutionEnd(Lazy<Timestamp>([this]() { return GetExecutionEnd(); }), utils);
	if (4 & types)
		ValidateActive(Lazy<bool>([this]() { return GetActive(); }), utils);
}

void ObjectImpl<CheckResult>::SimpleValidateCommand(const Lazy<Value>& value, const ValidationUtils& utils)
{
	if (value().IsObjectType<Function>()) {
		Function::Ptr func = value();
		if (func->IsDeprecated())
			Log(LogWarning, "CheckResult") << "Attribute 'command' for object '" << dynamic_cast<ConfigObject *>(this)->GetName() << "' of type '" << dynamic_cast<ConfigObject *>(this)->GetReflectionType()->GetName() << "' is set to a deprecated function: " << func->GetName();
	}

}

void ObjectImpl<CheckResult>::SimpleValidateOutput(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateCheckSource(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateTtl(const Lazy<double>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidatePerformanceData(const Lazy<Array::Ptr>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateVarsBefore(const Lazy<Dictionary::Ptr>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateVarsAfter(const Lazy<Dictionary::Ptr>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateExitStatus(const Lazy<int>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateState(const Lazy<ServiceState>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateScheduleStart(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateScheduleEnd(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateExecutionStart(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateExecutionEnd(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<CheckResult>::SimpleValidateActive(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

ObjectImpl<CheckResult>::ObjectImpl()
{
	SetCommand(GetDefaultCommand(), true);
	SetOutput(GetDefaultOutput(), true);
	SetCheckSource(GetDefaultCheckSource(), true);
	SetTtl(GetDefaultTtl(), true);
	SetPerformanceData(GetDefaultPerformanceData(), true);
	SetVarsBefore(GetDefaultVarsBefore(), true);
	SetVarsAfter(GetDefaultVarsAfter(), true);
	SetExitStatus(GetDefaultExitStatus(), true);
	SetState(GetDefaultState(), true);
	SetScheduleStart(GetDefaultScheduleStart(), true);
	SetScheduleEnd(GetDefaultScheduleEnd(), true);
	SetExecutionStart(GetDefaultExecutionStart(), true);
	SetExecutionEnd(GetDefaultExecutionEnd(), true);
	SetActive(GetDefaultActive(), true);
}

ObjectImpl<CheckResult>::~ObjectImpl()
{ }

void ObjectImpl<CheckResult>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	switch (id) {
		case 0:
			SetCommand(value, suppress_events, cookie);
			break;
		case 1:
			SetOutput(value, suppress_events, cookie);
			break;
		case 2:
			SetCheckSource(value, suppress_events, cookie);
			break;
		case 3:
			SetTtl(value, suppress_events, cookie);
			break;
		case 4:
			SetPerformanceData(value, suppress_events, cookie);
			break;
		case 5:
			SetVarsBefore(value, suppress_events, cookie);
			break;
		case 6:
			SetVarsAfter(value, suppress_events, cookie);
			break;
		case 7:
			SetExitStatus(value, suppress_events, cookie);
			break;
		case 8:
			SetState(static_cast<ServiceState>(static_cast<int>(value)), suppress_events, cookie);
			break;
		case 9:
			SetScheduleStart(value, suppress_events, cookie);
			break;
		case 10:
			SetScheduleEnd(value, suppress_events, cookie);
			break;
		case 11:
			SetExecutionStart(value, suppress_events, cookie);
			break;
		case 12:
			SetExecutionEnd(value, suppress_events, cookie);
			break;
		case 13:
			SetActive(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<CheckResult>::GetField(int id) const
{
	switch (id) {
		case 0:
			return GetCommand();
		case 1:
			return GetOutput();
		case 2:
			return GetCheckSource();
		case 3:
			return GetTtl();
		case 4:
			return GetPerformanceData();
		case 5:
			return GetVarsBefore();
		case 6:
			return GetVarsAfter();
		case 7:
			return GetExitStatus();
		case 8:
			return GetState();
		case 9:
			return GetScheduleStart();
		case 10:
			return GetScheduleEnd();
		case 11:
			return GetExecutionStart();
		case 12:
			return GetExecutionEnd();
		case 13:
			return GetActive();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<CheckResult>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	switch (id) {
		case 0:
			ValidateCommand(lvalue, utils);
			break;
		case 1:
			ValidateOutput(lvalue, utils);
			break;
		case 2:
			ValidateCheckSource(lvalue, utils);
			break;
		case 3:
			ValidateTtl(lvalue, utils);
			break;
		case 4:
			ValidatePerformanceData(lvalue, utils);
			break;
		case 5:
			ValidateVarsBefore(lvalue, utils);
			break;
		case 6:
			ValidateVarsAfter(lvalue, utils);
			break;
		case 7:
			ValidateExitStatus(lvalue, utils);
			break;
		case 8:
			ValidateState(static_cast<Lazy<ServiceState> >(static_cast<Lazy<int> >(lvalue)), utils);
			break;
		case 9:
			ValidateScheduleStart(lvalue, utils);
			break;
		case 10:
			ValidateScheduleEnd(lvalue, utils);
			break;
		case 11:
			ValidateExecutionStart(lvalue, utils);
			break;
		case 12:
			ValidateExecutionEnd(lvalue, utils);
			break;
		case 13:
			ValidateActive(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<CheckResult>::NotifyField(int id, const Value& cookie)
{
	switch (id) {
		case 0:
			NotifyCommand(cookie);
			break;
		case 1:
			NotifyOutput(cookie);
			break;
		case 2:
			NotifyCheckSource(cookie);
			break;
		case 3:
			NotifyTtl(cookie);
			break;
		case 4:
			NotifyPerformanceData(cookie);
			break;
		case 5:
			NotifyVarsBefore(cookie);
			break;
		case 6:
			NotifyVarsAfter(cookie);
			break;
		case 7:
			NotifyExitStatus(cookie);
			break;
		case 8:
			NotifyState(cookie);
			break;
		case 9:
			NotifyScheduleStart(cookie);
			break;
		case 10:
			NotifyScheduleEnd(cookie);
			break;
		case 11:
			NotifyExecutionStart(cookie);
			break;
		case 12:
			NotifyExecutionEnd(cookie);
			break;
		case 13:
			NotifyActive(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<CheckResult>::NavigateField(int id) const
{
	throw std::runtime_error("Invalid field ID.");
}

Value ObjectImpl<CheckResult>::GetCommand() const
{
	return m_Command;
}

String ObjectImpl<CheckResult>::GetOutput() const
{
	return m_Output;
}

String ObjectImpl<CheckResult>::GetCheckSource() const
{
	return m_CheckSource;
}

double ObjectImpl<CheckResult>::GetTtl() const
{
	return m_Ttl;
}

Array::Ptr ObjectImpl<CheckResult>::GetPerformanceData() const
{
	return m_PerformanceData;
}

Dictionary::Ptr ObjectImpl<CheckResult>::GetVarsBefore() const
{
	return m_VarsBefore;
}

Dictionary::Ptr ObjectImpl<CheckResult>::GetVarsAfter() const
{
	return m_VarsAfter;
}

int ObjectImpl<CheckResult>::GetExitStatus() const
{
	return m_ExitStatus;
}

ServiceState ObjectImpl<CheckResult>::GetState() const
{
	return m_State;
}

Timestamp ObjectImpl<CheckResult>::GetScheduleStart() const
{
	return m_ScheduleStart;
}

Timestamp ObjectImpl<CheckResult>::GetScheduleEnd() const
{
	return m_ScheduleEnd;
}

Timestamp ObjectImpl<CheckResult>::GetExecutionStart() const
{
	return m_ExecutionStart;
}

Timestamp ObjectImpl<CheckResult>::GetExecutionEnd() const
{
	return m_ExecutionEnd;
}

bool ObjectImpl<CheckResult>::GetActive() const
{
	return m_Active;
}

void ObjectImpl<CheckResult>::SetCommand(const Value& value, bool suppress_events, const Value& cookie)
{
	m_Command = value;
	if (!suppress_events)
		NotifyCommand(cookie);
}

void ObjectImpl<CheckResult>::SetOutput(const String& value, bool suppress_events, const Value& cookie)
{
	m_Output = value;
	if (!suppress_events)
		NotifyOutput(cookie);
}

void ObjectImpl<CheckResult>::SetCheckSource(const String& value, bool suppress_events, const Value& cookie)
{
	m_CheckSource = value;
	if (!suppress_events)
		NotifyCheckSource(cookie);
}

void ObjectImpl<CheckResult>::SetTtl(double value, bool suppress_events, const Value& cookie)
{
	m_Ttl = value;
	if (!suppress_events)
		NotifyTtl(cookie);
}

void ObjectImpl<CheckResult>::SetPerformanceData(const Array::Ptr& value, bool suppress_events, const Value& cookie)
{
	m_PerformanceData = value;
	if (!suppress_events)
		NotifyPerformanceData(cookie);
}

void ObjectImpl<CheckResult>::SetVarsBefore(const Dictionary::Ptr& value, bool suppress_events, const Value& cookie)
{
	m_VarsBefore = value;
	if (!suppress_events)
		NotifyVarsBefore(cookie);
}

void ObjectImpl<CheckResult>::SetVarsAfter(const Dictionary::Ptr& value, bool suppress_events, const Value& cookie)
{
	m_VarsAfter = value;
	if (!suppress_events)
		NotifyVarsAfter(cookie);
}

void ObjectImpl<CheckResult>::SetExitStatus(int value, bool suppress_events, const Value& cookie)
{
	m_ExitStatus = value;
	if (!suppress_events)
		NotifyExitStatus(cookie);
}

void ObjectImpl<CheckResult>::SetState(const ServiceState& value, bool suppress_events, const Value& cookie)
{
	m_State = value;
	if (!suppress_events)
		NotifyState(cookie);
}

void ObjectImpl<CheckResult>::SetScheduleStart(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_ScheduleStart = value;
	if (!suppress_events)
		NotifyScheduleStart(cookie);
}

void ObjectImpl<CheckResult>::SetScheduleEnd(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_ScheduleEnd = value;
	if (!suppress_events)
		NotifyScheduleEnd(cookie);
}

void ObjectImpl<CheckResult>::SetExecutionStart(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_ExecutionStart = value;
	if (!suppress_events)
		NotifyExecutionStart(cookie);
}

void ObjectImpl<CheckResult>::SetExecutionEnd(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_ExecutionEnd = value;
	if (!suppress_events)
		NotifyExecutionEnd(cookie);
}

void ObjectImpl<CheckResult>::SetActive(bool value, bool suppress_events, const Value& cookie)
{
	m_Active = value;
	if (!suppress_events)
		NotifyActive(cookie);
}

void ObjectImpl<CheckResult>::NotifyCommand(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnCommandChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyOutput(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnOutputChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyCheckSource(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnCheckSourceChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyTtl(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnTtlChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyPerformanceData(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPerformanceDataChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyVarsBefore(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnVarsBeforeChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyVarsAfter(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnVarsAfterChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyExitStatus(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnExitStatusChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyState(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnStateChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyScheduleStart(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnScheduleStartChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyScheduleEnd(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnScheduleEndChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyExecutionStart(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnExecutionStartChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyExecutionEnd(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnExecutionEndChanged(static_cast<CheckResult *>(this), cookie);
}

void ObjectImpl<CheckResult>::NotifyActive(const Value& cookie)
{
	OnActiveChanged(static_cast<CheckResult *>(this), cookie);
}

Value ObjectImpl<CheckResult>::GetDefaultCommand() const
{
	return Value();
}

String ObjectImpl<CheckResult>::GetDefaultOutput() const
{
	return String();
}

String ObjectImpl<CheckResult>::GetDefaultCheckSource() const
{
	return String();
}

double ObjectImpl<CheckResult>::GetDefaultTtl() const
{
	return double();
}

Array::Ptr ObjectImpl<CheckResult>::GetDefaultPerformanceData() const
{
	return Array::Ptr();
}

Dictionary::Ptr ObjectImpl<CheckResult>::GetDefaultVarsBefore() const
{
	return Dictionary::Ptr();
}

Dictionary::Ptr ObjectImpl<CheckResult>::GetDefaultVarsAfter() const
{
	return Dictionary::Ptr();
}

int ObjectImpl<CheckResult>::GetDefaultExitStatus() const
{
	return int();
}

ServiceState ObjectImpl<CheckResult>::GetDefaultState() const
{
	return ServiceState();
}

Timestamp ObjectImpl<CheckResult>::GetDefaultScheduleStart() const
{
	return Timestamp();
}

Timestamp ObjectImpl<CheckResult>::GetDefaultScheduleEnd() const
{
	return Timestamp();
}

Timestamp ObjectImpl<CheckResult>::GetDefaultExecutionStart() const
{
	return Timestamp();
}

Timestamp ObjectImpl<CheckResult>::GetDefaultExecutionEnd() const
{
	return Timestamp();
}

bool ObjectImpl<CheckResult>::GetDefaultActive() const
{
	 return true; 
}


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnCommandChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnOutputChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnCheckSourceChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnTtlChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnPerformanceDataChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnVarsBeforeChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnVarsAfterChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnExitStatusChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnStateChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnScheduleStartChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnScheduleEndChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnExecutionStartChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnExecutionEndChanged;


boost::signals2::signal<void (const intrusive_ptr<CheckResult>&, const Value&)> ObjectImpl<CheckResult>::OnActiveChanged;

void ObjectImpl<CheckResult>::ValidateActive(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateActive(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateCheckSource(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateCheckSource(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateCommand(const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateCommand(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateExecutionEnd(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateExecutionEnd(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateExecutionStart(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateExecutionStart(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateExitStatus(const Lazy<int>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateExitStatus(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateOutput(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateOutput(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidatePerformanceData(const Lazy<Array::Ptr>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePerformanceData(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateScheduleEnd(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateScheduleEnd(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateScheduleStart(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateScheduleStart(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateState(const Lazy<ServiceState>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateState(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateTtl(const Lazy<double>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateTtl(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateVarsAfter(const Lazy<Dictionary::Ptr>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateVarsAfter(lvalue, utils);
}

void ObjectImpl<CheckResult>::ValidateVarsBefore(const Lazy<Dictionary::Ptr>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateVarsBefore(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
