#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<NotificationResult>::TypeImpl()
{ }

TypeImpl<NotificationResult>::~TypeImpl()
{ }

String TypeImpl<NotificationResult>::GetName() const
{
	return "NotificationResult";
}

int TypeImpl<NotificationResult>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<NotificationResult>::GetBaseType() const
{
	return Object::TypeInstance;
}

int TypeImpl<NotificationResult>::GetFieldId(const String& name) const
{
	int offset = 0;

	switch (static_cast<int>(Utility::SDBM(name, 1))) {
		case 97:
			if (name == "active")
				return offset + 6;

			break;
		case 99:
			if (name == "command")
				return offset + 0;

			break;
		case 101:
			if (name == "execution_endpoint")
				return offset + 2;
			if (name == "exit_status")
				return offset + 3;
			if (name == "execution_start")
				return offset + 4;
			if (name == "execution_end")
				return offset + 5;

			break;
		case 111:
			if (name == "output")
				return offset + 1;

			break;
	}

	return -1;
}

Field TypeImpl<NotificationResult>::GetFieldInfo(int id) const
{
	switch (id) {
		case 0:
			return {0, "Value", "command", "command", nullptr, 4, 0};
		case 1:
			return {1, "String", "output", "output", nullptr, 4, 0};
		case 2:
			return {2, "String", "execution_endpoint", "execution_endpoint", nullptr, 4, 0};
		case 3:
			return {3, "Number", "exit_status", "exit_status", nullptr, 4, 0};
		case 4:
			return {4, "Timestamp", "execution_start", "execution_start", nullptr, 4, 0};
		case 5:
			return {5, "Timestamp", "execution_end", "execution_end", nullptr, 4, 0};
		case 6:
			return {6, "Number", "active", "active", nullptr, 4, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<NotificationResult>::GetFieldCount() const
{
	return 7;
}

ObjectFactory TypeImpl<NotificationResult>::GetFactory() const
{
	return TypeHelper<NotificationResult, false>::GetFactory();
}

std::vector<String> TypeImpl<NotificationResult>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<NotificationResult>::GetActivationPriority() const
{
	return 0;
}

void TypeImpl<NotificationResult>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	switch (fieldId) {
		case 0:
			ObjectImpl<NotificationResult>::OnCommandChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<NotificationResult>::OnOutputChanged.connect(callback);
			break;
		case 2:
			ObjectImpl<NotificationResult>::OnExecutionEndpointChanged.connect(callback);
			break;
		case 3:
			ObjectImpl<NotificationResult>::OnExitStatusChanged.connect(callback);
			break;
		case 4:
			ObjectImpl<NotificationResult>::OnExecutionStartChanged.connect(callback);
			break;
		case 5:
			ObjectImpl<NotificationResult>::OnExecutionEndChanged.connect(callback);
			break;
		case 6:
			ObjectImpl<NotificationResult>::OnActiveChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<NotificationResult>::Validate(int types, const ValidationUtils& utils)
{
	if (4 & types)
		ValidateCommand(Lazy<Value>([this]() { return GetCommand(); }), utils);
	if (4 & types)
		ValidateOutput(Lazy<String>([this]() { return GetOutput(); }), utils);
	if (4 & types)
		ValidateExecutionEndpoint(Lazy<String>([this]() { return GetExecutionEndpoint(); }), utils);
	if (4 & types)
		ValidateExitStatus(Lazy<int>([this]() { return GetExitStatus(); }), utils);
	if (4 & types)
		ValidateExecutionStart(Lazy<Timestamp>([this]() { return GetExecutionStart(); }), utils);
	if (4 & types)
		ValidateExecutionEnd(Lazy<Timestamp>([this]() { return GetExecutionEnd(); }), utils);
	if (4 & types)
		ValidateActive(Lazy<bool>([this]() { return GetActive(); }), utils);
}

void ObjectImpl<NotificationResult>::SimpleValidateCommand(const Lazy<Value>& value, const ValidationUtils& utils)
{
	if (value().IsObjectType<Function>()) {
		Function::Ptr func = value();
		if (func->IsDeprecated())
			Log(LogWarning, "NotificationResult") << "Attribute 'command' for object '" << dynamic_cast<ConfigObject *>(this)->GetName() << "' of type '" << dynamic_cast<ConfigObject *>(this)->GetReflectionType()->GetName() << "' is set to a deprecated function: " << func->GetName();
	}

}

void ObjectImpl<NotificationResult>::SimpleValidateOutput(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<NotificationResult>::SimpleValidateExecutionEndpoint(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<NotificationResult>::SimpleValidateExitStatus(const Lazy<int>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<NotificationResult>::SimpleValidateExecutionStart(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<NotificationResult>::SimpleValidateExecutionEnd(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<NotificationResult>::SimpleValidateActive(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

ObjectImpl<NotificationResult>::ObjectImpl()
{
	SetCommand(GetDefaultCommand(), true);
	SetOutput(GetDefaultOutput(), true);
	SetExecutionEndpoint(GetDefaultExecutionEndpoint(), true);
	SetExitStatus(GetDefaultExitStatus(), true);
	SetExecutionStart(GetDefaultExecutionStart(), true);
	SetExecutionEnd(GetDefaultExecutionEnd(), true);
	SetActive(GetDefaultActive(), true);
}

ObjectImpl<NotificationResult>::~ObjectImpl()
{ }

void ObjectImpl<NotificationResult>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	switch (id) {
		case 0:
			SetCommand(value, suppress_events, cookie);
			break;
		case 1:
			SetOutput(value, suppress_events, cookie);
			break;
		case 2:
			SetExecutionEndpoint(value, suppress_events, cookie);
			break;
		case 3:
			SetExitStatus(value, suppress_events, cookie);
			break;
		case 4:
			SetExecutionStart(value, suppress_events, cookie);
			break;
		case 5:
			SetExecutionEnd(value, suppress_events, cookie);
			break;
		case 6:
			SetActive(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<NotificationResult>::GetField(int id) const
{
	switch (id) {
		case 0:
			return GetCommand();
		case 1:
			return GetOutput();
		case 2:
			return GetExecutionEndpoint();
		case 3:
			return GetExitStatus();
		case 4:
			return GetExecutionStart();
		case 5:
			return GetExecutionEnd();
		case 6:
			return GetActive();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<NotificationResult>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	switch (id) {
		case 0:
			ValidateCommand(lvalue, utils);
			break;
		case 1:
			ValidateOutput(lvalue, utils);
			break;
		case 2:
			ValidateExecutionEndpoint(lvalue, utils);
			break;
		case 3:
			ValidateExitStatus(lvalue, utils);
			break;
		case 4:
			ValidateExecutionStart(lvalue, utils);
			break;
		case 5:
			ValidateExecutionEnd(lvalue, utils);
			break;
		case 6:
			ValidateActive(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<NotificationResult>::NotifyField(int id, const Value& cookie)
{
	switch (id) {
		case 0:
			NotifyCommand(cookie);
			break;
		case 1:
			NotifyOutput(cookie);
			break;
		case 2:
			NotifyExecutionEndpoint(cookie);
			break;
		case 3:
			NotifyExitStatus(cookie);
			break;
		case 4:
			NotifyExecutionStart(cookie);
			break;
		case 5:
			NotifyExecutionEnd(cookie);
			break;
		case 6:
			NotifyActive(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<NotificationResult>::NavigateField(int id) const
{
	throw std::runtime_error("Invalid field ID.");
}

Value ObjectImpl<NotificationResult>::GetCommand() const
{
	return m_Command;
}

String ObjectImpl<NotificationResult>::GetOutput() const
{
	return m_Output;
}

String ObjectImpl<NotificationResult>::GetExecutionEndpoint() const
{
	return m_ExecutionEndpoint;
}

int ObjectImpl<NotificationResult>::GetExitStatus() const
{
	return m_ExitStatus;
}

Timestamp ObjectImpl<NotificationResult>::GetExecutionStart() const
{
	return m_ExecutionStart;
}

Timestamp ObjectImpl<NotificationResult>::GetExecutionEnd() const
{
	return m_ExecutionEnd;
}

bool ObjectImpl<NotificationResult>::GetActive() const
{
	return m_Active;
}

void ObjectImpl<NotificationResult>::SetCommand(const Value& value, bool suppress_events, const Value& cookie)
{
	m_Command = value;
	if (!suppress_events)
		NotifyCommand(cookie);
}

void ObjectImpl<NotificationResult>::SetOutput(const String& value, bool suppress_events, const Value& cookie)
{
	m_Output = value;
	if (!suppress_events)
		NotifyOutput(cookie);
}

void ObjectImpl<NotificationResult>::SetExecutionEndpoint(const String& value, bool suppress_events, const Value& cookie)
{
	m_ExecutionEndpoint = value;
	if (!suppress_events)
		NotifyExecutionEndpoint(cookie);
}

void ObjectImpl<NotificationResult>::SetExitStatus(int value, bool suppress_events, const Value& cookie)
{
	m_ExitStatus = value;
	if (!suppress_events)
		NotifyExitStatus(cookie);
}

void ObjectImpl<NotificationResult>::SetExecutionStart(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_ExecutionStart = value;
	if (!suppress_events)
		NotifyExecutionStart(cookie);
}

void ObjectImpl<NotificationResult>::SetExecutionEnd(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_ExecutionEnd = value;
	if (!suppress_events)
		NotifyExecutionEnd(cookie);
}

void ObjectImpl<NotificationResult>::SetActive(bool value, bool suppress_events, const Value& cookie)
{
	m_Active = value;
	if (!suppress_events)
		NotifyActive(cookie);
}

void ObjectImpl<NotificationResult>::NotifyCommand(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnCommandChanged(static_cast<NotificationResult *>(this), cookie);
}

void ObjectImpl<NotificationResult>::NotifyOutput(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnOutputChanged(static_cast<NotificationResult *>(this), cookie);
}

void ObjectImpl<NotificationResult>::NotifyExecutionEndpoint(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnExecutionEndpointChanged(static_cast<NotificationResult *>(this), cookie);
}

void ObjectImpl<NotificationResult>::NotifyExitStatus(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnExitStatusChanged(static_cast<NotificationResult *>(this), cookie);
}

void ObjectImpl<NotificationResult>::NotifyExecutionStart(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnExecutionStartChanged(static_cast<NotificationResult *>(this), cookie);
}

void ObjectImpl<NotificationResult>::NotifyExecutionEnd(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnExecutionEndChanged(static_cast<NotificationResult *>(this), cookie);
}

void ObjectImpl<NotificationResult>::NotifyActive(const Value& cookie)
{
	OnActiveChanged(static_cast<NotificationResult *>(this), cookie);
}

Value ObjectImpl<NotificationResult>::GetDefaultCommand() const
{
	return Value();
}

String ObjectImpl<NotificationResult>::GetDefaultOutput() const
{
	return String();
}

String ObjectImpl<NotificationResult>::GetDefaultExecutionEndpoint() const
{
	return String();
}

int ObjectImpl<NotificationResult>::GetDefaultExitStatus() const
{
	return int();
}

Timestamp ObjectImpl<NotificationResult>::GetDefaultExecutionStart() const
{
	return Timestamp();
}

Timestamp ObjectImpl<NotificationResult>::GetDefaultExecutionEnd() const
{
	return Timestamp();
}

bool ObjectImpl<NotificationResult>::GetDefaultActive() const
{
	 return true; 
}


boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> ObjectImpl<NotificationResult>::OnCommandChanged;


boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> ObjectImpl<NotificationResult>::OnOutputChanged;


boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> ObjectImpl<NotificationResult>::OnExecutionEndpointChanged;


boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> ObjectImpl<NotificationResult>::OnExitStatusChanged;


boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> ObjectImpl<NotificationResult>::OnExecutionStartChanged;


boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> ObjectImpl<NotificationResult>::OnExecutionEndChanged;


boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> ObjectImpl<NotificationResult>::OnActiveChanged;

void ObjectImpl<NotificationResult>::ValidateActive(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateActive(lvalue, utils);
}

void ObjectImpl<NotificationResult>::ValidateCommand(const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateCommand(lvalue, utils);
}

void ObjectImpl<NotificationResult>::ValidateExecutionEnd(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateExecutionEnd(lvalue, utils);
}

void ObjectImpl<NotificationResult>::ValidateExecutionEndpoint(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateExecutionEndpoint(lvalue, utils);
}

void ObjectImpl<NotificationResult>::ValidateExecutionStart(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateExecutionStart(lvalue, utils);
}

void ObjectImpl<NotificationResult>::ValidateExitStatus(const Lazy<int>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateExitStatus(lvalue, utils);
}

void ObjectImpl<NotificationResult>::ValidateOutput(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateOutput(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
