#ifndef NOTIFICATIONRESULT_TI
#define NOTIFICATIONRESULT_TI

#include "base/object.hpp"
#include "base/type.hpp"
#include "base/value.hpp"
#include "base/array.hpp"
#include "base/dictionary.hpp"
#include <boost/signals2.hpp>

namespace icinga
{

class NotificationResult;

template<>
class TypeImpl<NotificationResult> : public Type
{
public:
	DECLARE_PTR_TYPEDEFS(TypeImpl<NotificationResult>);

	TypeImpl();
	~TypeImpl() override;

	String GetName() const override;
	int GetAttributes() const override;
	Type::Ptr GetBaseType() const override;
	int GetFieldId(const String& name) const override;
	Field GetFieldInfo(int id) const override;
	int GetFieldCount() const override;
	ObjectFactory GetFactory() const override;
	std::vector<String> GetLoadDependencies() const override;
	int GetActivationPriority() const override;
public:
	void RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback) override;
};


template<>
class ObjectImpl<NotificationResult> : public Object
{
public:
	DECLARE_PTR_TYPEDEFS(ObjectImpl<NotificationResult>);

	void Validate(int types, const ValidationUtils& utils) override;
	void SimpleValidateCommand(const Lazy<Value>& value, const ValidationUtils& utils);
	void SimpleValidateOutput(const Lazy<String>& value, const ValidationUtils& utils);
	void SimpleValidateExecutionEndpoint(const Lazy<String>& value, const ValidationUtils& utils);
	void SimpleValidateExitStatus(const Lazy<int>& value, const ValidationUtils& utils);
	void SimpleValidateExecutionStart(const Lazy<Timestamp>& value, const ValidationUtils& utils);
	void SimpleValidateExecutionEnd(const Lazy<Timestamp>& value, const ValidationUtils& utils);
	void SimpleValidateActive(const Lazy<bool>& value, const ValidationUtils& utils);
public:
	ObjectImpl<NotificationResult>();
public:
	~ObjectImpl<NotificationResult>() override;
public:
	void SetField(int id, const Value& value, bool suppress_events = false, const Value& cookie = Empty) override;
public:
	Value GetField(int id) const override;
public:
	void ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils) override;
public:
	void NotifyField(int id, const Value& cookie = Empty) override;
public:
	Object::Ptr NavigateField(int id) const override;
public:
	Value GetCommand() const;
public:
	String GetOutput() const;
public:
	String GetExecutionEndpoint() const;
public:
	int GetExitStatus() const;
public:
	Timestamp GetExecutionStart() const;
public:
	Timestamp GetExecutionEnd() const;
public:
	bool GetActive() const;
public:
	void SetCommand(const Value& value, bool suppress_events = false, const Value& cookie = Empty);
public:
	void SetOutput(const String& value, bool suppress_events = false, const Value& cookie = Empty);
public:
	void SetExecutionEndpoint(const String& value, bool suppress_events = false, const Value& cookie = Empty);
public:
	void SetExitStatus(int value, bool suppress_events = false, const Value& cookie = Empty);
public:
	void SetExecutionStart(const Timestamp& value, bool suppress_events = false, const Value& cookie = Empty);
public:
	void SetExecutionEnd(const Timestamp& value, bool suppress_events = false, const Value& cookie = Empty);
public:
	void SetActive(bool value, bool suppress_events = false, const Value& cookie = Empty);
protected:
public:
	virtual void NotifyCommand(const Value& cookie = Empty);
public:
	virtual void NotifyOutput(const Value& cookie = Empty);
public:
	virtual void NotifyExecutionEndpoint(const Value& cookie = Empty);
public:
	virtual void NotifyExitStatus(const Value& cookie = Empty);
public:
	virtual void NotifyExecutionStart(const Value& cookie = Empty);
public:
	virtual void NotifyExecutionEnd(const Value& cookie = Empty);
public:
	virtual void NotifyActive(const Value& cookie = Empty);
private:
	inline Value GetDefaultCommand() const;
private:
	inline String GetDefaultOutput() const;
private:
	inline String GetDefaultExecutionEndpoint() const;
private:
	inline int GetDefaultExitStatus() const;
private:
	inline Timestamp GetDefaultExecutionStart() const;
private:
	inline Timestamp GetDefaultExecutionEnd() const;
private:
	inline bool GetDefaultActive() const;
protected:
	virtual void ValidateCommand(const Lazy<Value>& lvalue, const ValidationUtils& utils);
protected:
	virtual void ValidateOutput(const Lazy<String>& lvalue, const ValidationUtils& utils);
protected:
	virtual void ValidateExecutionEndpoint(const Lazy<String>& lvalue, const ValidationUtils& utils);
protected:
	virtual void ValidateExitStatus(const Lazy<int>& lvalue, const ValidationUtils& utils);
protected:
	virtual void ValidateExecutionStart(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils);
protected:
	virtual void ValidateExecutionEnd(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils);
protected:
	virtual void ValidateActive(const Lazy<bool>& lvalue, const ValidationUtils& utils);
private:
	Value m_Command;
	String m_Output;
	String m_ExecutionEndpoint;
	int m_ExitStatus;
	Timestamp m_ExecutionStart;
	Timestamp m_ExecutionEnd;
	bool m_Active;
public:
	static boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> OnCommandChanged;
	static boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> OnOutputChanged;
	static boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> OnExecutionEndpointChanged;
	static boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> OnExitStatusChanged;
	static boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> OnExecutionStartChanged;
	static boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> OnExecutionEndChanged;
	static boost::signals2::signal<void (const intrusive_ptr<NotificationResult>&, const Value&)> OnActiveChanged;
};

}
#endif /* NOTIFICATIONRESULT_TI */
