#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<InfluxdbWriter>::TypeImpl()
{ }

TypeImpl<InfluxdbWriter>::~TypeImpl()
{ }

String TypeImpl<InfluxdbWriter>::GetName() const
{
	return "InfluxdbWriter";
}

int TypeImpl<InfluxdbWriter>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<InfluxdbWriter>::GetBaseType() const
{
	return ConfigObject::TypeInstance;
}

int TypeImpl<InfluxdbWriter>::GetFieldId(const String& name) const
{
	int offset = ConfigObject::TypeInstance->GetFieldCount();

	switch (static_cast<int>(Utility::SDBM(name, 5))) {
		case -1852654458:
			if (name == "password")
				return offset + 4;

			break;
		case -1844041471:
			if (name == "port")
				return offset + 1;

			break;
		case -1622403728:
			if (name == "ssl_ca_cert")
				return offset + 5;
			if (name == "ssl_cert")
				return offset + 6;

			break;
		case -1622403726:
			if (name == "ssl_enable")
				return offset + 12;

			break;
		case -1622403720:
			if (name == "ssl_key")
				return offset + 7;

			break;
		case -350347868:
			if (name == "flush_interval")
				return offset + 10;
			if (name == "flush_threshold")
				return offset + 11;

			break;
		case -95083038:
			if (name == "enable_send_thresholds")
				return offset + 13;
			if (name == "enable_send_metadata")
				return offset + 14;
			if (name == "enable_ha")
				return offset + 15;

			break;
		case -40882409:
			if (name == "host_template")
				return offset + 8;

			break;
		case 385355187:
			if (name == "service_template")
				return offset + 9;

			break;
		case 501261128:
			if (name == "host")
				return offset + 0;

			break;
		case 600140707:
			if (name == "username")
				return offset + 3;

			break;
		case 1659523960:
			if (name == "database")
				return offset + 2;

			break;
	}

	return ConfigObject::TypeInstance->GetFieldId(name);
}

Field TypeImpl<InfluxdbWriter>::GetFieldInfo(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount();
	if (real_id < 0) { return ConfigObject::TypeInstance->GetFieldInfo(id); }
	switch (real_id) {
		case 0:
			return {0, "String", "host", "host", nullptr, 258, 0};
		case 1:
			return {1, "String", "port", "port", nullptr, 258, 0};
		case 2:
			return {2, "String", "database", "database", nullptr, 258, 0};
		case 3:
			return {3, "String", "username", "username", nullptr, 2, 0};
		case 4:
			return {4, "String", "password", "password", nullptr, 2050, 0};
		case 5:
			return {5, "String", "ssl_ca_cert", "ssl_ca_cert", nullptr, 2, 0};
		case 6:
			return {6, "String", "ssl_cert", "ssl_cert", nullptr, 2, 0};
		case 7:
			return {7, "String", "ssl_key", "ssl_key", nullptr, 2, 0};
		case 8:
			return {8, "Dictionary", "host_template", "host_template", nullptr, 258, 0};
		case 9:
			return {9, "Dictionary", "service_template", "service_template", nullptr, 258, 0};
		case 10:
			return {10, "Number", "flush_interval", "flush_interval", nullptr, 2, 0};
		case 11:
			return {11, "Number", "flush_threshold", "flush_threshold", nullptr, 2, 0};
		case 12:
			return {12, "Number", "ssl_enable", "ssl_enable", nullptr, 2, 0};
		case 13:
			return {13, "Number", "enable_send_thresholds", "enable_send_thresholds", nullptr, 2, 0};
		case 14:
			return {14, "Number", "enable_send_metadata", "enable_send_metadata", nullptr, 2, 0};
		case 15:
			return {15, "Number", "enable_ha", "enable_ha", nullptr, 2, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<InfluxdbWriter>::GetFieldCount() const
{
	return 16 + ConfigObject::TypeInstance->GetFieldCount();
}

ObjectFactory TypeImpl<InfluxdbWriter>::GetFactory() const
{
	return TypeHelper<InfluxdbWriter, false>::GetFactory();
}

std::vector<String> TypeImpl<InfluxdbWriter>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<InfluxdbWriter>::GetActivationPriority() const
{
	return 100;
}

void TypeImpl<InfluxdbWriter>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	int real_id = fieldId - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::TypeInstance->RegisterAttributeHandler(fieldId, callback); return; }
	switch (real_id) {
		case 0:
			ObjectImpl<InfluxdbWriter>::OnHostChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<InfluxdbWriter>::OnPortChanged.connect(callback);
			break;
		case 2:
			ObjectImpl<InfluxdbWriter>::OnDatabaseChanged.connect(callback);
			break;
		case 3:
			ObjectImpl<InfluxdbWriter>::OnUsernameChanged.connect(callback);
			break;
		case 4:
			ObjectImpl<InfluxdbWriter>::OnPasswordChanged.connect(callback);
			break;
		case 5:
			ObjectImpl<InfluxdbWriter>::OnSslCaCertChanged.connect(callback);
			break;
		case 6:
			ObjectImpl<InfluxdbWriter>::OnSslCertChanged.connect(callback);
			break;
		case 7:
			ObjectImpl<InfluxdbWriter>::OnSslKeyChanged.connect(callback);
			break;
		case 8:
			ObjectImpl<InfluxdbWriter>::OnHostTemplateChanged.connect(callback);
			break;
		case 9:
			ObjectImpl<InfluxdbWriter>::OnServiceTemplateChanged.connect(callback);
			break;
		case 10:
			ObjectImpl<InfluxdbWriter>::OnFlushIntervalChanged.connect(callback);
			break;
		case 11:
			ObjectImpl<InfluxdbWriter>::OnFlushThresholdChanged.connect(callback);
			break;
		case 12:
			ObjectImpl<InfluxdbWriter>::OnSslEnableChanged.connect(callback);
			break;
		case 13:
			ObjectImpl<InfluxdbWriter>::OnEnableSendThresholdsChanged.connect(callback);
			break;
		case 14:
			ObjectImpl<InfluxdbWriter>::OnEnableSendMetadataChanged.connect(callback);
			break;
		case 15:
			ObjectImpl<InfluxdbWriter>::OnEnableHaChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<InfluxdbWriter>::Validate(int types, const ValidationUtils& utils)
{
	ConfigObject::Validate(types, utils);

	if (2 & types)
		ValidateHost(Lazy<String>([this]() { return GetHost(); }), utils);
	if (2 & types)
		ValidatePort(Lazy<String>([this]() { return GetPort(); }), utils);
	if (2 & types)
		ValidateDatabase(Lazy<String>([this]() { return GetDatabase(); }), utils);
	if (2 & types)
		ValidateUsername(Lazy<String>([this]() { return GetUsername(); }), utils);
	if (2 & types)
		ValidatePassword(Lazy<String>([this]() { return GetPassword(); }), utils);
	if (2 & types)
		ValidateSslCaCert(Lazy<String>([this]() { return GetSslCaCert(); }), utils);
	if (2 & types)
		ValidateSslCert(Lazy<String>([this]() { return GetSslCert(); }), utils);
	if (2 & types)
		ValidateSslKey(Lazy<String>([this]() { return GetSslKey(); }), utils);
	if (2 & types)
		ValidateHostTemplate(Lazy<Dictionary::Ptr>([this]() { return GetHostTemplate(); }), utils);
	if (2 & types)
		ValidateServiceTemplate(Lazy<Dictionary::Ptr>([this]() { return GetServiceTemplate(); }), utils);
	if (2 & types)
		ValidateFlushInterval(Lazy<int>([this]() { return GetFlushInterval(); }), utils);
	if (2 & types)
		ValidateFlushThreshold(Lazy<int>([this]() { return GetFlushThreshold(); }), utils);
	if (2 & types)
		ValidateSslEnable(Lazy<bool>([this]() { return GetSslEnable(); }), utils);
	if (2 & types)
		ValidateEnableSendThresholds(Lazy<bool>([this]() { return GetEnableSendThresholds(); }), utils);
	if (2 & types)
		ValidateEnableSendMetadata(Lazy<bool>([this]() { return GetEnableSendMetadata(); }), utils);
	if (2 & types)
		ValidateEnableHa(Lazy<bool>([this]() { return GetEnableHa(); }), utils);
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateHost(const Lazy<String>& value, const ValidationUtils& utils)
{
	if (value().IsEmpty())
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), { "host" }, "Attribute must not be empty."));

}

void ObjectImpl<InfluxdbWriter>::SimpleValidatePort(const Lazy<String>& value, const ValidationUtils& utils)
{
	if (value().IsEmpty())
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), { "port" }, "Attribute must not be empty."));

}

void ObjectImpl<InfluxdbWriter>::SimpleValidateDatabase(const Lazy<String>& value, const ValidationUtils& utils)
{
	if (value().IsEmpty())
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), { "database" }, "Attribute must not be empty."));

}

void ObjectImpl<InfluxdbWriter>::SimpleValidateUsername(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidatePassword(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateSslCaCert(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateSslCert(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateSslKey(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateHostTemplate(const Lazy<Dictionary::Ptr>& value, const ValidationUtils& utils)
{
	if (!value())
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), { "host_template" }, "Attribute must not be empty."));

}

void ObjectImpl<InfluxdbWriter>::SimpleValidateServiceTemplate(const Lazy<Dictionary::Ptr>& value, const ValidationUtils& utils)
{
	if (!value())
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), { "service_template" }, "Attribute must not be empty."));

}

void ObjectImpl<InfluxdbWriter>::SimpleValidateFlushInterval(const Lazy<int>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateFlushThreshold(const Lazy<int>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateSslEnable(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateEnableSendThresholds(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateEnableSendMetadata(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<InfluxdbWriter>::SimpleValidateEnableHa(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

ObjectImpl<InfluxdbWriter>::ObjectImpl()
{
	SetHost(GetDefaultHost(), true);
	SetPort(GetDefaultPort(), true);
	SetDatabase(GetDefaultDatabase(), true);
	SetUsername(GetDefaultUsername(), true);
	SetPassword(GetDefaultPassword(), true);
	SetSslCaCert(GetDefaultSslCaCert(), true);
	SetSslCert(GetDefaultSslCert(), true);
	SetSslKey(GetDefaultSslKey(), true);
	SetHostTemplate(GetDefaultHostTemplate(), true);
	SetServiceTemplate(GetDefaultServiceTemplate(), true);
	SetFlushInterval(GetDefaultFlushInterval(), true);
	SetFlushThreshold(GetDefaultFlushThreshold(), true);
	SetSslEnable(GetDefaultSslEnable(), true);
	SetEnableSendThresholds(GetDefaultEnableSendThresholds(), true);
	SetEnableSendMetadata(GetDefaultEnableSendMetadata(), true);
	SetEnableHa(GetDefaultEnableHa(), true);
}

ObjectImpl<InfluxdbWriter>::~ObjectImpl()
{ }

void ObjectImpl<InfluxdbWriter>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::SetField(id, value, suppress_events, cookie); return; }
	switch (real_id) {
		case 0:
			SetHost(value, suppress_events, cookie);
			break;
		case 1:
			SetPort(value, suppress_events, cookie);
			break;
		case 2:
			SetDatabase(value, suppress_events, cookie);
			break;
		case 3:
			SetUsername(value, suppress_events, cookie);
			break;
		case 4:
			SetPassword(value, suppress_events, cookie);
			break;
		case 5:
			SetSslCaCert(value, suppress_events, cookie);
			break;
		case 6:
			SetSslCert(value, suppress_events, cookie);
			break;
		case 7:
			SetSslKey(value, suppress_events, cookie);
			break;
		case 8:
			SetHostTemplate(value, suppress_events, cookie);
			break;
		case 9:
			SetServiceTemplate(value, suppress_events, cookie);
			break;
		case 10:
			SetFlushInterval(value, suppress_events, cookie);
			break;
		case 11:
			SetFlushThreshold(value, suppress_events, cookie);
			break;
		case 12:
			SetSslEnable(value, suppress_events, cookie);
			break;
		case 13:
			SetEnableSendThresholds(value, suppress_events, cookie);
			break;
		case 14:
			SetEnableSendMetadata(value, suppress_events, cookie);
			break;
		case 15:
			SetEnableHa(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<InfluxdbWriter>::GetField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::GetField(id); }
	switch (real_id) {
		case 0:
			return GetHost();
		case 1:
			return GetPort();
		case 2:
			return GetDatabase();
		case 3:
			return GetUsername();
		case 4:
			return GetPassword();
		case 5:
			return GetSslCaCert();
		case 6:
			return GetSslCert();
		case 7:
			return GetSslKey();
		case 8:
			return GetHostTemplate();
		case 9:
			return GetServiceTemplate();
		case 10:
			return GetFlushInterval();
		case 11:
			return GetFlushThreshold();
		case 12:
			return GetSslEnable();
		case 13:
			return GetEnableSendThresholds();
		case 14:
			return GetEnableSendMetadata();
		case 15:
			return GetEnableHa();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<InfluxdbWriter>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::ValidateField(id, lvalue, utils); return; }
	switch (real_id) {
		case 0:
			ValidateHost(lvalue, utils);
			break;
		case 1:
			ValidatePort(lvalue, utils);
			break;
		case 2:
			ValidateDatabase(lvalue, utils);
			break;
		case 3:
			ValidateUsername(lvalue, utils);
			break;
		case 4:
			ValidatePassword(lvalue, utils);
			break;
		case 5:
			ValidateSslCaCert(lvalue, utils);
			break;
		case 6:
			ValidateSslCert(lvalue, utils);
			break;
		case 7:
			ValidateSslKey(lvalue, utils);
			break;
		case 8:
			ValidateHostTemplate(lvalue, utils);
			break;
		case 9:
			ValidateServiceTemplate(lvalue, utils);
			break;
		case 10:
			ValidateFlushInterval(lvalue, utils);
			break;
		case 11:
			ValidateFlushThreshold(lvalue, utils);
			break;
		case 12:
			ValidateSslEnable(lvalue, utils);
			break;
		case 13:
			ValidateEnableSendThresholds(lvalue, utils);
			break;
		case 14:
			ValidateEnableSendMetadata(lvalue, utils);
			break;
		case 15:
			ValidateEnableHa(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<InfluxdbWriter>::NotifyField(int id, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::NotifyField(id, cookie); return; }
	switch (real_id) {
		case 0:
			NotifyHost(cookie);
			break;
		case 1:
			NotifyPort(cookie);
			break;
		case 2:
			NotifyDatabase(cookie);
			break;
		case 3:
			NotifyUsername(cookie);
			break;
		case 4:
			NotifyPassword(cookie);
			break;
		case 5:
			NotifySslCaCert(cookie);
			break;
		case 6:
			NotifySslCert(cookie);
			break;
		case 7:
			NotifySslKey(cookie);
			break;
		case 8:
			NotifyHostTemplate(cookie);
			break;
		case 9:
			NotifyServiceTemplate(cookie);
			break;
		case 10:
			NotifyFlushInterval(cookie);
			break;
		case 11:
			NotifyFlushThreshold(cookie);
			break;
		case 12:
			NotifySslEnable(cookie);
			break;
		case 13:
			NotifyEnableSendThresholds(cookie);
			break;
		case 14:
			NotifyEnableSendMetadata(cookie);
			break;
		case 15:
			NotifyEnableHa(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<InfluxdbWriter>::NavigateField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::NavigateField(id); }
	throw std::runtime_error("Invalid field ID.");
}

String ObjectImpl<InfluxdbWriter>::GetHost() const
{
	return m_Host;
}

String ObjectImpl<InfluxdbWriter>::GetPort() const
{
	return m_Port;
}

String ObjectImpl<InfluxdbWriter>::GetDatabase() const
{
	return m_Database;
}

String ObjectImpl<InfluxdbWriter>::GetUsername() const
{
	return m_Username;
}

String ObjectImpl<InfluxdbWriter>::GetPassword() const
{
	return m_Password;
}

String ObjectImpl<InfluxdbWriter>::GetSslCaCert() const
{
	return m_SslCaCert;
}

String ObjectImpl<InfluxdbWriter>::GetSslCert() const
{
	return m_SslCert;
}

String ObjectImpl<InfluxdbWriter>::GetSslKey() const
{
	return m_SslKey;
}

Dictionary::Ptr ObjectImpl<InfluxdbWriter>::GetHostTemplate() const
{
	return m_HostTemplate;
}

Dictionary::Ptr ObjectImpl<InfluxdbWriter>::GetServiceTemplate() const
{
	return m_ServiceTemplate;
}

int ObjectImpl<InfluxdbWriter>::GetFlushInterval() const
{
	return m_FlushInterval;
}

int ObjectImpl<InfluxdbWriter>::GetFlushThreshold() const
{
	return m_FlushThreshold;
}

bool ObjectImpl<InfluxdbWriter>::GetSslEnable() const
{
	return m_SslEnable;
}

bool ObjectImpl<InfluxdbWriter>::GetEnableSendThresholds() const
{
	return m_EnableSendThresholds;
}

bool ObjectImpl<InfluxdbWriter>::GetEnableSendMetadata() const
{
	return m_EnableSendMetadata;
}

bool ObjectImpl<InfluxdbWriter>::GetEnableHa() const
{
	return m_EnableHa;
}

void ObjectImpl<InfluxdbWriter>::SetHost(const String& value, bool suppress_events, const Value& cookie)
{
	m_Host = value;
	if (!suppress_events)
		NotifyHost(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetPort(const String& value, bool suppress_events, const Value& cookie)
{
	m_Port = value;
	if (!suppress_events)
		NotifyPort(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetDatabase(const String& value, bool suppress_events, const Value& cookie)
{
	m_Database = value;
	if (!suppress_events)
		NotifyDatabase(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetUsername(const String& value, bool suppress_events, const Value& cookie)
{
	m_Username = value;
	if (!suppress_events)
		NotifyUsername(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetPassword(const String& value, bool suppress_events, const Value& cookie)
{
	m_Password = value;
	if (!suppress_events)
		NotifyPassword(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetSslCaCert(const String& value, bool suppress_events, const Value& cookie)
{
	m_SslCaCert = value;
	if (!suppress_events)
		NotifySslCaCert(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetSslCert(const String& value, bool suppress_events, const Value& cookie)
{
	m_SslCert = value;
	if (!suppress_events)
		NotifySslCert(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetSslKey(const String& value, bool suppress_events, const Value& cookie)
{
	m_SslKey = value;
	if (!suppress_events)
		NotifySslKey(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetHostTemplate(const Dictionary::Ptr& value, bool suppress_events, const Value& cookie)
{
	m_HostTemplate = value;
	if (!suppress_events)
		NotifyHostTemplate(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetServiceTemplate(const Dictionary::Ptr& value, bool suppress_events, const Value& cookie)
{
	m_ServiceTemplate = value;
	if (!suppress_events)
		NotifyServiceTemplate(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetFlushInterval(int value, bool suppress_events, const Value& cookie)
{
	m_FlushInterval = value;
	if (!suppress_events)
		NotifyFlushInterval(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetFlushThreshold(int value, bool suppress_events, const Value& cookie)
{
	m_FlushThreshold = value;
	if (!suppress_events)
		NotifyFlushThreshold(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetSslEnable(bool value, bool suppress_events, const Value& cookie)
{
	m_SslEnable = value;
	if (!suppress_events)
		NotifySslEnable(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetEnableSendThresholds(bool value, bool suppress_events, const Value& cookie)
{
	m_EnableSendThresholds = value;
	if (!suppress_events)
		NotifyEnableSendThresholds(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetEnableSendMetadata(bool value, bool suppress_events, const Value& cookie)
{
	m_EnableSendMetadata = value;
	if (!suppress_events)
		NotifyEnableSendMetadata(cookie);
}

void ObjectImpl<InfluxdbWriter>::SetEnableHa(bool value, bool suppress_events, const Value& cookie)
{
	m_EnableHa = value;
	if (!suppress_events)
		NotifyEnableHa(cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyHost(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnHostChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyPort(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPortChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyDatabase(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnDatabaseChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyUsername(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnUsernameChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyPassword(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPasswordChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifySslCaCert(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnSslCaCertChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifySslCert(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnSslCertChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifySslKey(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnSslKeyChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyHostTemplate(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnHostTemplateChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyServiceTemplate(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnServiceTemplateChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyFlushInterval(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnFlushIntervalChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyFlushThreshold(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnFlushThresholdChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifySslEnable(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnSslEnableChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyEnableSendThresholds(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnEnableSendThresholdsChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyEnableSendMetadata(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnEnableSendMetadataChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

void ObjectImpl<InfluxdbWriter>::NotifyEnableHa(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnEnableHaChanged(static_cast<InfluxdbWriter *>(this), cookie);
}

String ObjectImpl<InfluxdbWriter>::GetDefaultHost() const
{
	 return "127.0.0.1"; 
}

String ObjectImpl<InfluxdbWriter>::GetDefaultPort() const
{
	 return "8086"; 
}

String ObjectImpl<InfluxdbWriter>::GetDefaultDatabase() const
{
	 return "icinga2"; 
}

String ObjectImpl<InfluxdbWriter>::GetDefaultUsername() const
{
	 return ""; 
}

String ObjectImpl<InfluxdbWriter>::GetDefaultPassword() const
{
	 return ""; 
}

String ObjectImpl<InfluxdbWriter>::GetDefaultSslCaCert() const
{
	 return ""; 
}

String ObjectImpl<InfluxdbWriter>::GetDefaultSslCert() const
{
	 return ""; 
}

String ObjectImpl<InfluxdbWriter>::GetDefaultSslKey() const
{
	 return ""; 
}

Dictionary::Ptr ObjectImpl<InfluxdbWriter>::GetDefaultHostTemplate() const
{
	
			return new Dictionary({
				{ "measurement", "$host.check_command$" },
				{ "tags", new Dictionary({
					{ "hostname", "$host.name$" }
				}) }
			});
		
}

Dictionary::Ptr ObjectImpl<InfluxdbWriter>::GetDefaultServiceTemplate() const
{
	
			return new Dictionary({
				{ "measurement", "$service.check_command$" },
				{ "tags", new Dictionary({
					{ "hostname", "$host.name$" },
					{ "service", "$service.name$" }
				}) }
			});
		
}

int ObjectImpl<InfluxdbWriter>::GetDefaultFlushInterval() const
{
	 return 10; 
}

int ObjectImpl<InfluxdbWriter>::GetDefaultFlushThreshold() const
{
	 return 1024; 
}

bool ObjectImpl<InfluxdbWriter>::GetDefaultSslEnable() const
{
	 return false; 
}

bool ObjectImpl<InfluxdbWriter>::GetDefaultEnableSendThresholds() const
{
	 return false; 
}

bool ObjectImpl<InfluxdbWriter>::GetDefaultEnableSendMetadata() const
{
	 return false; 
}

bool ObjectImpl<InfluxdbWriter>::GetDefaultEnableHa() const
{
	 return false; 
}


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnHostChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnPortChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnDatabaseChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnUsernameChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnPasswordChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnSslCaCertChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnSslCertChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnSslKeyChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnHostTemplateChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnServiceTemplateChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnFlushIntervalChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnFlushThresholdChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnSslEnableChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnEnableSendThresholdsChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnEnableSendMetadataChanged;


boost::signals2::signal<void (const intrusive_ptr<InfluxdbWriter>&, const Value&)> ObjectImpl<InfluxdbWriter>::OnEnableHaChanged;

static void TIValidateInfluxdbWriter_1_2(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& key, const Value& value, std::vector<String>& location, const ValidationUtils& utils)
{
	bool known_attribute = false;
	do {
		known_attribute = true;
		if (value.IsEmpty() || value.IsScalar())
			return;
	} while (0);

	if (!known_attribute)
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Invalid attribute: " + key));
	else
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Invalid type."));
}

static void TIValidateInfluxdbWriter_1(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& key, const Value& value, std::vector<String>& location, const ValidationUtils& utils)
{
	bool known_attribute = false;
	do {
		if (key != "measurement")
			break;
		known_attribute = true;
		if (value.IsEmpty() || value.IsScalar())
			return;
	} while (0);

	do {
		if (key != "tags")
			break;
		known_attribute = true;
		if (value.IsObjectType<Dictionary>()) {
			Dictionary::Ptr dict = value;
			{
				ObjectLock olock(dict);
				for (const Dictionary::Pair& kv : dict) {
					const String& akey = kv.first;
					const Value& avalue = kv.second;
					location.emplace_back(akey);
					TIValidateInfluxdbWriter_1_2(object, akey, avalue, location, utils);
					location.pop_back();
				}
			}
			return;
		}
	} while (0);

	if (!known_attribute)
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Invalid attribute: " + key));
	else
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Invalid type."));
}

static void TIValidateInfluxdbWriter_2_2(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& key, const Value& value, std::vector<String>& location, const ValidationUtils& utils)
{
	bool known_attribute = false;
	do {
		known_attribute = true;
		if (value.IsEmpty() || value.IsScalar())
			return;
	} while (0);

	if (!known_attribute)
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Invalid attribute: " + key));
	else
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Invalid type."));
}

static void TIValidateInfluxdbWriter_2(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& key, const Value& value, std::vector<String>& location, const ValidationUtils& utils)
{
	bool known_attribute = false;
	do {
		if (key != "measurement")
			break;
		known_attribute = true;
		if (value.IsEmpty() || value.IsScalar())
			return;
	} while (0);

	do {
		if (key != "tags")
			break;
		known_attribute = true;
		if (value.IsObjectType<Dictionary>()) {
			Dictionary::Ptr dict = value;
			{
				ObjectLock olock(dict);
				for (const Dictionary::Pair& kv : dict) {
					const String& akey = kv.first;
					const Value& avalue = kv.second;
					location.emplace_back(akey);
					TIValidateInfluxdbWriter_2_2(object, akey, avalue, location, utils);
					location.pop_back();
				}
			}
			return;
		}
	} while (0);

	if (!known_attribute)
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Invalid attribute: " + key));
	else
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Invalid type."));
}

static void TIValidateInfluxdbWriterDatabase(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (value.IsEmpty())
		return;

}

static void TIValidateInfluxdbWriterEnableHa(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, bool value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (!value)
		return;

}

static void TIValidateInfluxdbWriterEnableSendMetadata(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, bool value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (!value)
		return;

}

static void TIValidateInfluxdbWriterEnableSendThresholds(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, bool value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (!value)
		return;

}

static void TIValidateInfluxdbWriterFlushInterval(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, int value, std::vector<String>& location, const ValidationUtils& utils)
{
}

static void TIValidateInfluxdbWriterFlushThreshold(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, int value, std::vector<String>& location, const ValidationUtils& utils)
{
}

static void TIValidateInfluxdbWriterHost(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (value.IsEmpty())
		return;

}

static void TIValidateInfluxdbWriterHostTemplate(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const Dictionary::Ptr& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (!value)
		return;

	do {
		const Dictionary::Ptr& dict = value;
		{
			ObjectLock olock(dict);
			for (const Dictionary::Pair& kv : dict) {
				const String& akey = kv.first;
				const Value& avalue = kv.second;
				location.emplace_back(akey);
				TIValidateInfluxdbWriter_1(object, akey, avalue, location, utils);
				location.pop_back();
			}
		}
		if (dict->Get("measurement").IsEmpty())
			BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Required dictionary item 'measurement' is not set."));
		return;
	} while (0);

}

static void TIValidateInfluxdbWriterPassword(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (value.IsEmpty())
		return;

}

static void TIValidateInfluxdbWriterPort(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (value.IsEmpty())
		return;

}

static void TIValidateInfluxdbWriterServiceTemplate(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const Dictionary::Ptr& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (!value)
		return;

	do {
		const Dictionary::Ptr& dict = value;
		{
			ObjectLock olock(dict);
			for (const Dictionary::Pair& kv : dict) {
				const String& akey = kv.first;
				const Value& avalue = kv.second;
				location.emplace_back(akey);
				TIValidateInfluxdbWriter_2(object, akey, avalue, location, utils);
				location.pop_back();
			}
		}
		if (dict->Get("measurement").IsEmpty())
			BOOST_THROW_EXCEPTION(ValidationError(dynamic_pointer_cast<ConfigObject>(object), location, "Required dictionary item 'measurement' is not set."));
		return;
	} while (0);

}

static void TIValidateInfluxdbWriterSslCaCert(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (value.IsEmpty())
		return;

}

static void TIValidateInfluxdbWriterSslCert(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (value.IsEmpty())
		return;

}

static void TIValidateInfluxdbWriterSslEnable(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, bool value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (!value)
		return;

}

static void TIValidateInfluxdbWriterSslKey(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (value.IsEmpty())
		return;

}

static void TIValidateInfluxdbWriterUsername(const intrusive_ptr<ObjectImpl<InfluxdbWriter> >& object, const String& value, std::vector<String>& location, const ValidationUtils& utils)
{
	if (value.IsEmpty())
		return;

}

void ObjectImpl<InfluxdbWriter>::ValidateDatabase(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateDatabase(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("database");
	TIValidateInfluxdbWriterDatabase(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateEnableHa(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateEnableHa(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("enable_ha");
	TIValidateInfluxdbWriterEnableHa(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateEnableSendMetadata(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateEnableSendMetadata(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("enable_send_metadata");
	TIValidateInfluxdbWriterEnableSendMetadata(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateEnableSendThresholds(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateEnableSendThresholds(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("enable_send_thresholds");
	TIValidateInfluxdbWriterEnableSendThresholds(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateFlushInterval(const Lazy<int>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateFlushInterval(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("flush_interval");
	TIValidateInfluxdbWriterFlushInterval(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateFlushThreshold(const Lazy<int>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateFlushThreshold(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("flush_threshold");
	TIValidateInfluxdbWriterFlushThreshold(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateHost(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateHost(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("host");
	TIValidateInfluxdbWriterHost(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateHostTemplate(const Lazy<Dictionary::Ptr>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateHostTemplate(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("host_template");
	TIValidateInfluxdbWriterHostTemplate(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidatePassword(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePassword(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("password");
	TIValidateInfluxdbWriterPassword(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidatePort(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePort(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("port");
	TIValidateInfluxdbWriterPort(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateServiceTemplate(const Lazy<Dictionary::Ptr>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateServiceTemplate(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("service_template");
	TIValidateInfluxdbWriterServiceTemplate(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateSslCaCert(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateSslCaCert(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("ssl_ca_cert");
	TIValidateInfluxdbWriterSslCaCert(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateSslCert(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateSslCert(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("ssl_cert");
	TIValidateInfluxdbWriterSslCert(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateSslEnable(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateSslEnable(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("ssl_enable");
	TIValidateInfluxdbWriterSslEnable(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateSslKey(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateSslKey(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("ssl_key");
	TIValidateInfluxdbWriterSslKey(this, lvalue(), location, utils);
	location.pop_back();
}

void ObjectImpl<InfluxdbWriter>::ValidateUsername(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateUsername(lvalue, utils);
	std::vector<String> location;
	location.emplace_back("username");
	TIValidateInfluxdbWriterUsername(this, lvalue(), location, utils);
	location.pop_back();
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
