#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<OpenTsdbWriter>::TypeImpl()
{ }

TypeImpl<OpenTsdbWriter>::~TypeImpl()
{ }

String TypeImpl<OpenTsdbWriter>::GetName() const
{
	return "OpenTsdbWriter";
}

int TypeImpl<OpenTsdbWriter>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<OpenTsdbWriter>::GetBaseType() const
{
	return ConfigObject::TypeInstance;
}

int TypeImpl<OpenTsdbWriter>::GetFieldId(const String& name) const
{
	int offset = ConfigObject::TypeInstance->GetFieldCount();

	switch (static_cast<int>(Utility::SDBM(name, 1))) {
		case 99:
			if (name == "connected")
				return offset + 3;

			break;
		case 101:
			if (name == "enable_ha")
				return offset + 2;

			break;
		case 104:
			if (name == "host")
				return offset + 0;

			break;
		case 112:
			if (name == "port")
				return offset + 1;

			break;
		case 115:
			if (name == "should_connect")
				return offset + 4;

			break;
	}

	return ConfigObject::TypeInstance->GetFieldId(name);
}

Field TypeImpl<OpenTsdbWriter>::GetFieldInfo(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount();
	if (real_id < 0) { return ConfigObject::TypeInstance->GetFieldInfo(id); }
	switch (real_id) {
		case 0:
			return {0, "String", "host", "host", nullptr, 2, 0};
		case 1:
			return {1, "String", "port", "port", nullptr, 2, 0};
		case 2:
			return {2, "Number", "enable_ha", "enable_ha", nullptr, 2, 0};
		case 3:
			return {3, "Number", "connected", "connected", nullptr, 1025, 0};
		case 4:
			return {4, "Number", "should_connect", "should_connect", nullptr, 1025, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<OpenTsdbWriter>::GetFieldCount() const
{
	return 5 + ConfigObject::TypeInstance->GetFieldCount();
}

ObjectFactory TypeImpl<OpenTsdbWriter>::GetFactory() const
{
	return TypeHelper<OpenTsdbWriter, false>::GetFactory();
}

std::vector<String> TypeImpl<OpenTsdbWriter>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<OpenTsdbWriter>::GetActivationPriority() const
{
	return 100;
}

void TypeImpl<OpenTsdbWriter>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	int real_id = fieldId - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::TypeInstance->RegisterAttributeHandler(fieldId, callback); return; }
	switch (real_id) {
		case 0:
			ObjectImpl<OpenTsdbWriter>::OnHostChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<OpenTsdbWriter>::OnPortChanged.connect(callback);
			break;
		case 2:
			ObjectImpl<OpenTsdbWriter>::OnEnableHaChanged.connect(callback);
			break;
		case 3:
			ObjectImpl<OpenTsdbWriter>::OnConnectedChanged.connect(callback);
			break;
		case 4:
			ObjectImpl<OpenTsdbWriter>::OnShouldConnectChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<OpenTsdbWriter>::Validate(int types, const ValidationUtils& utils)
{
	ConfigObject::Validate(types, utils);

	if (2 & types)
		ValidateHost(Lazy<String>([this]() { return GetHost(); }), utils);
	if (2 & types)
		ValidatePort(Lazy<String>([this]() { return GetPort(); }), utils);
	if (2 & types)
		ValidateEnableHa(Lazy<bool>([this]() { return GetEnableHa(); }), utils);
	if (1 & types)
		ValidateConnected(Lazy<bool>([this]() { return GetConnected(); }), utils);
	if (1 & types)
		ValidateShouldConnect(Lazy<bool>([this]() { return GetShouldConnect(); }), utils);
}

void ObjectImpl<OpenTsdbWriter>::SimpleValidateHost(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<OpenTsdbWriter>::SimpleValidatePort(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<OpenTsdbWriter>::SimpleValidateEnableHa(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<OpenTsdbWriter>::SimpleValidateConnected(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<OpenTsdbWriter>::SimpleValidateShouldConnect(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

ObjectImpl<OpenTsdbWriter>::ObjectImpl()
{
	SetHost(GetDefaultHost(), true);
	SetPort(GetDefaultPort(), true);
	SetEnableHa(GetDefaultEnableHa(), true);
	SetConnected(GetDefaultConnected(), true);
	SetShouldConnect(GetDefaultShouldConnect(), true);
}

ObjectImpl<OpenTsdbWriter>::~ObjectImpl()
{ }

void ObjectImpl<OpenTsdbWriter>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::SetField(id, value, suppress_events, cookie); return; }
	switch (real_id) {
		case 0:
			SetHost(value, suppress_events, cookie);
			break;
		case 1:
			SetPort(value, suppress_events, cookie);
			break;
		case 2:
			SetEnableHa(value, suppress_events, cookie);
			break;
		case 3:
			SetConnected(value, suppress_events, cookie);
			break;
		case 4:
			SetShouldConnect(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<OpenTsdbWriter>::GetField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::GetField(id); }
	switch (real_id) {
		case 0:
			return GetHost();
		case 1:
			return GetPort();
		case 2:
			return GetEnableHa();
		case 3:
			return GetConnected();
		case 4:
			return GetShouldConnect();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<OpenTsdbWriter>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::ValidateField(id, lvalue, utils); return; }
	switch (real_id) {
		case 0:
			ValidateHost(lvalue, utils);
			break;
		case 1:
			ValidatePort(lvalue, utils);
			break;
		case 2:
			ValidateEnableHa(lvalue, utils);
			break;
		case 3:
			ValidateConnected(lvalue, utils);
			break;
		case 4:
			ValidateShouldConnect(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<OpenTsdbWriter>::NotifyField(int id, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::NotifyField(id, cookie); return; }
	switch (real_id) {
		case 0:
			NotifyHost(cookie);
			break;
		case 1:
			NotifyPort(cookie);
			break;
		case 2:
			NotifyEnableHa(cookie);
			break;
		case 3:
			NotifyConnected(cookie);
			break;
		case 4:
			NotifyShouldConnect(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<OpenTsdbWriter>::NavigateField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::NavigateField(id); }
	throw std::runtime_error("Invalid field ID.");
}

String ObjectImpl<OpenTsdbWriter>::GetHost() const
{
	return m_Host;
}

String ObjectImpl<OpenTsdbWriter>::GetPort() const
{
	return m_Port;
}

bool ObjectImpl<OpenTsdbWriter>::GetEnableHa() const
{
	return m_EnableHa;
}

bool ObjectImpl<OpenTsdbWriter>::GetConnected() const
{
	return m_Connected;
}

bool ObjectImpl<OpenTsdbWriter>::GetShouldConnect() const
{
	return m_ShouldConnect;
}

void ObjectImpl<OpenTsdbWriter>::SetHost(const String& value, bool suppress_events, const Value& cookie)
{
	m_Host = value;
	if (!suppress_events)
		NotifyHost(cookie);
}

void ObjectImpl<OpenTsdbWriter>::SetPort(const String& value, bool suppress_events, const Value& cookie)
{
	m_Port = value;
	if (!suppress_events)
		NotifyPort(cookie);
}

void ObjectImpl<OpenTsdbWriter>::SetEnableHa(bool value, bool suppress_events, const Value& cookie)
{
	m_EnableHa = value;
	if (!suppress_events)
		NotifyEnableHa(cookie);
}

void ObjectImpl<OpenTsdbWriter>::SetConnected(bool value, bool suppress_events, const Value& cookie)
{
	m_Connected = value;
	if (!suppress_events)
		NotifyConnected(cookie);
}

void ObjectImpl<OpenTsdbWriter>::SetShouldConnect(bool value, bool suppress_events, const Value& cookie)
{
	m_ShouldConnect = value;
	if (!suppress_events)
		NotifyShouldConnect(cookie);
}

void ObjectImpl<OpenTsdbWriter>::NotifyHost(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnHostChanged(static_cast<OpenTsdbWriter *>(this), cookie);
}

void ObjectImpl<OpenTsdbWriter>::NotifyPort(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPortChanged(static_cast<OpenTsdbWriter *>(this), cookie);
}

void ObjectImpl<OpenTsdbWriter>::NotifyEnableHa(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnEnableHaChanged(static_cast<OpenTsdbWriter *>(this), cookie);
}

void ObjectImpl<OpenTsdbWriter>::NotifyConnected(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnConnectedChanged(static_cast<OpenTsdbWriter *>(this), cookie);
}

void ObjectImpl<OpenTsdbWriter>::NotifyShouldConnect(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnShouldConnectChanged(static_cast<OpenTsdbWriter *>(this), cookie);
}

String ObjectImpl<OpenTsdbWriter>::GetDefaultHost() const
{
	 return "127.0.0.1"; 
}

String ObjectImpl<OpenTsdbWriter>::GetDefaultPort() const
{
	 return "4242"; 
}

bool ObjectImpl<OpenTsdbWriter>::GetDefaultEnableHa() const
{
	 return false; 
}

bool ObjectImpl<OpenTsdbWriter>::GetDefaultConnected() const
{
	return bool();
}

bool ObjectImpl<OpenTsdbWriter>::GetDefaultShouldConnect() const
{
	 return true; 
}


boost::signals2::signal<void (const intrusive_ptr<OpenTsdbWriter>&, const Value&)> ObjectImpl<OpenTsdbWriter>::OnHostChanged;


boost::signals2::signal<void (const intrusive_ptr<OpenTsdbWriter>&, const Value&)> ObjectImpl<OpenTsdbWriter>::OnPortChanged;


boost::signals2::signal<void (const intrusive_ptr<OpenTsdbWriter>&, const Value&)> ObjectImpl<OpenTsdbWriter>::OnEnableHaChanged;


boost::signals2::signal<void (const intrusive_ptr<OpenTsdbWriter>&, const Value&)> ObjectImpl<OpenTsdbWriter>::OnConnectedChanged;


boost::signals2::signal<void (const intrusive_ptr<OpenTsdbWriter>&, const Value&)> ObjectImpl<OpenTsdbWriter>::OnShouldConnectChanged;

void ObjectImpl<OpenTsdbWriter>::ValidateConnected(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateConnected(lvalue, utils);
}

void ObjectImpl<OpenTsdbWriter>::ValidateEnableHa(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateEnableHa(lvalue, utils);
}

void ObjectImpl<OpenTsdbWriter>::ValidateHost(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateHost(lvalue, utils);
}

void ObjectImpl<OpenTsdbWriter>::ValidatePort(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePort(lvalue, utils);
}

void ObjectImpl<OpenTsdbWriter>::ValidateShouldConnect(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateShouldConnect(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
