#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<Endpoint>::TypeImpl()
{ }

TypeImpl<Endpoint>::~TypeImpl()
{ }

String TypeImpl<Endpoint>::GetName() const
{
	return "Endpoint";
}

int TypeImpl<Endpoint>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<Endpoint>::GetBaseType() const
{
	return ConfigObject::TypeInstance;
}

int TypeImpl<Endpoint>::GetFieldId(const String& name) const
{
	int offset = ConfigObject::TypeInstance->GetFieldCount();

	switch (static_cast<int>(Utility::SDBM(name, 3))) {
		case 817565085:
			if (name == "bytes_sent_per_second")
				return offset + 5;
			if (name == "bytes_received_per_second")
				return offset + 6;

			break;
		case 825170594:
			if (name == "connecting")
				return offset + 11;
			if (name == "connected")
				return offset + 13;

			break;
		case 866478124:
			if (name == "host")
				return offset + 0;

			break;
		case 898605758:
			if (name == "last_message_sent")
				return offset + 9;
			if (name == "last_message_received")
				return offset + 10;

			break;
		case 899524128:
			if (name == "local_log_position")
				return offset + 7;

			break;
		case 899524132:
			if (name == "log_duration")
				return offset + 2;

			break;
		case 907129659:
			if (name == "messages_sent_per_second")
				return offset + 3;
			if (name == "messages_received_per_second")
				return offset + 4;

			break;
		case 932570163:
			if (name == "port")
				return offset + 1;

			break;
		case 948437178:
			if (name == "remote_log_position")
				return offset + 8;

			break;
		case 958010664:
			if (name == "syncing")
				return offset + 12;

			break;
	}

	return ConfigObject::TypeInstance->GetFieldId(name);
}

Field TypeImpl<Endpoint>::GetFieldInfo(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount();
	if (real_id < 0) { return ConfigObject::TypeInstance->GetFieldInfo(id); }
	switch (real_id) {
		case 0:
			return {0, "String", "host", "host", nullptr, 2, 0};
		case 1:
			return {1, "String", "port", "port", nullptr, 258, 0};
		case 2:
			return {2, "Number", "log_duration", "log_duration", nullptr, 2, 0};
		case 3:
			return {3, "Number", "messages_sent_per_second", "messages_sent_per_second", nullptr, 1089, 0};
		case 4:
			return {4, "Number", "messages_received_per_second", "messages_received_per_second", nullptr, 1089, 0};
		case 5:
			return {5, "Number", "bytes_sent_per_second", "bytes_sent_per_second", nullptr, 1089, 0};
		case 6:
			return {6, "Number", "bytes_received_per_second", "bytes_received_per_second", nullptr, 1089, 0};
		case 7:
			return {7, "Timestamp", "local_log_position", "local_log_position", nullptr, 4, 0};
		case 8:
			return {8, "Timestamp", "remote_log_position", "remote_log_position", nullptr, 4, 0};
		case 9:
			return {9, "Timestamp", "last_message_sent", "last_message_sent", nullptr, 1, 0};
		case 10:
			return {10, "Timestamp", "last_message_received", "last_message_received", nullptr, 1, 0};
		case 11:
			return {11, "Number", "connecting", "connecting", nullptr, 1025, 0};
		case 12:
			return {12, "Number", "syncing", "syncing", nullptr, 1025, 0};
		case 13:
			return {13, "Number", "connected", "connected", nullptr, 1089, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<Endpoint>::GetFieldCount() const
{
	return 14 + ConfigObject::TypeInstance->GetFieldCount();
}

ObjectFactory TypeImpl<Endpoint>::GetFactory() const
{
	return TypeHelper<Endpoint, false>::GetFactory();
}

std::vector<String> TypeImpl<Endpoint>::GetLoadDependencies() const
{
	std::vector<String> deps;
	deps.emplace_back("Zone");
	return deps;
}

int TypeImpl<Endpoint>::GetActivationPriority() const
{
	return 0;
}

void TypeImpl<Endpoint>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	int real_id = fieldId - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::TypeInstance->RegisterAttributeHandler(fieldId, callback); return; }
	switch (real_id) {
		case 0:
			ObjectImpl<Endpoint>::OnHostChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<Endpoint>::OnPortChanged.connect(callback);
			break;
		case 2:
			ObjectImpl<Endpoint>::OnLogDurationChanged.connect(callback);
			break;
		case 3:
			ObjectImpl<Endpoint>::OnMessagesSentPerSecondChanged.connect(callback);
			break;
		case 4:
			ObjectImpl<Endpoint>::OnMessagesReceivedPerSecondChanged.connect(callback);
			break;
		case 5:
			ObjectImpl<Endpoint>::OnBytesSentPerSecondChanged.connect(callback);
			break;
		case 6:
			ObjectImpl<Endpoint>::OnBytesReceivedPerSecondChanged.connect(callback);
			break;
		case 7:
			ObjectImpl<Endpoint>::OnLocalLogPositionChanged.connect(callback);
			break;
		case 8:
			ObjectImpl<Endpoint>::OnRemoteLogPositionChanged.connect(callback);
			break;
		case 9:
			ObjectImpl<Endpoint>::OnLastMessageSentChanged.connect(callback);
			break;
		case 10:
			ObjectImpl<Endpoint>::OnLastMessageReceivedChanged.connect(callback);
			break;
		case 11:
			ObjectImpl<Endpoint>::OnConnectingChanged.connect(callback);
			break;
		case 12:
			ObjectImpl<Endpoint>::OnSyncingChanged.connect(callback);
			break;
		case 13:
			ObjectImpl<Endpoint>::OnConnectedChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<Endpoint>::Validate(int types, const ValidationUtils& utils)
{
	ConfigObject::Validate(types, utils);

	if (2 & types)
		ValidateHost(Lazy<String>([this]() { return GetHost(); }), utils);
	if (2 & types)
		ValidatePort(Lazy<String>([this]() { return GetPort(); }), utils);
	if (2 & types)
		ValidateLogDuration(Lazy<double>([this]() { return GetLogDuration(); }), utils);
	if (1 & types)
		ValidateMessagesSentPerSecond(Lazy<double>([this]() { return GetMessagesSentPerSecond(); }), utils);
	if (1 & types)
		ValidateMessagesReceivedPerSecond(Lazy<double>([this]() { return GetMessagesReceivedPerSecond(); }), utils);
	if (1 & types)
		ValidateBytesSentPerSecond(Lazy<double>([this]() { return GetBytesSentPerSecond(); }), utils);
	if (1 & types)
		ValidateBytesReceivedPerSecond(Lazy<double>([this]() { return GetBytesReceivedPerSecond(); }), utils);
	if (4 & types)
		ValidateLocalLogPosition(Lazy<Timestamp>([this]() { return GetLocalLogPosition(); }), utils);
	if (4 & types)
		ValidateRemoteLogPosition(Lazy<Timestamp>([this]() { return GetRemoteLogPosition(); }), utils);
	if (1 & types)
		ValidateLastMessageSent(Lazy<Timestamp>([this]() { return GetLastMessageSent(); }), utils);
	if (1 & types)
		ValidateLastMessageReceived(Lazy<Timestamp>([this]() { return GetLastMessageReceived(); }), utils);
	if (1 & types)
		ValidateConnecting(Lazy<bool>([this]() { return GetConnecting(); }), utils);
	if (1 & types)
		ValidateSyncing(Lazy<bool>([this]() { return GetSyncing(); }), utils);
	if (1 & types)
		ValidateConnected(Lazy<bool>([this]() { return GetConnected(); }), utils);
}

void ObjectImpl<Endpoint>::SimpleValidateHost(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidatePort(const Lazy<String>& value, const ValidationUtils& utils)
{
	if (value().IsEmpty())
		BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), { "port" }, "Attribute must not be empty."));

}

void ObjectImpl<Endpoint>::SimpleValidateLogDuration(const Lazy<double>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateMessagesSentPerSecond(const Lazy<double>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateMessagesReceivedPerSecond(const Lazy<double>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateBytesSentPerSecond(const Lazy<double>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateBytesReceivedPerSecond(const Lazy<double>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateLocalLogPosition(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateRemoteLogPosition(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateLastMessageSent(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateLastMessageReceived(const Lazy<Timestamp>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateConnecting(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateSyncing(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<Endpoint>::SimpleValidateConnected(const Lazy<bool>& value, const ValidationUtils& utils)
{
}

ObjectImpl<Endpoint>::ObjectImpl()
{
	SetHost(GetDefaultHost(), true);
	SetPort(GetDefaultPort(), true);
	SetLogDuration(GetDefaultLogDuration(), true);
	SetMessagesSentPerSecond(GetDefaultMessagesSentPerSecond(), true);
	SetMessagesReceivedPerSecond(GetDefaultMessagesReceivedPerSecond(), true);
	SetBytesSentPerSecond(GetDefaultBytesSentPerSecond(), true);
	SetBytesReceivedPerSecond(GetDefaultBytesReceivedPerSecond(), true);
	SetLocalLogPosition(GetDefaultLocalLogPosition(), true);
	SetRemoteLogPosition(GetDefaultRemoteLogPosition(), true);
	SetLastMessageSent(GetDefaultLastMessageSent(), true);
	SetLastMessageReceived(GetDefaultLastMessageReceived(), true);
	SetConnecting(GetDefaultConnecting(), true);
	SetSyncing(GetDefaultSyncing(), true);
	SetConnected(GetDefaultConnected(), true);
}

ObjectImpl<Endpoint>::~ObjectImpl()
{ }

void ObjectImpl<Endpoint>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::SetField(id, value, suppress_events, cookie); return; }
	switch (real_id) {
		case 0:
			SetHost(value, suppress_events, cookie);
			break;
		case 1:
			SetPort(value, suppress_events, cookie);
			break;
		case 2:
			SetLogDuration(value, suppress_events, cookie);
			break;
		case 3:
			SetMessagesSentPerSecond(value, suppress_events, cookie);
			break;
		case 4:
			SetMessagesReceivedPerSecond(value, suppress_events, cookie);
			break;
		case 5:
			SetBytesSentPerSecond(value, suppress_events, cookie);
			break;
		case 6:
			SetBytesReceivedPerSecond(value, suppress_events, cookie);
			break;
		case 7:
			SetLocalLogPosition(value, suppress_events, cookie);
			break;
		case 8:
			SetRemoteLogPosition(value, suppress_events, cookie);
			break;
		case 9:
			SetLastMessageSent(value, suppress_events, cookie);
			break;
		case 10:
			SetLastMessageReceived(value, suppress_events, cookie);
			break;
		case 11:
			SetConnecting(value, suppress_events, cookie);
			break;
		case 12:
			SetSyncing(value, suppress_events, cookie);
			break;
		case 13:
			SetConnected(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<Endpoint>::GetField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::GetField(id); }
	switch (real_id) {
		case 0:
			return GetHost();
		case 1:
			return GetPort();
		case 2:
			return GetLogDuration();
		case 3:
			return GetMessagesSentPerSecond();
		case 4:
			return GetMessagesReceivedPerSecond();
		case 5:
			return GetBytesSentPerSecond();
		case 6:
			return GetBytesReceivedPerSecond();
		case 7:
			return GetLocalLogPosition();
		case 8:
			return GetRemoteLogPosition();
		case 9:
			return GetLastMessageSent();
		case 10:
			return GetLastMessageReceived();
		case 11:
			return GetConnecting();
		case 12:
			return GetSyncing();
		case 13:
			return GetConnected();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<Endpoint>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::ValidateField(id, lvalue, utils); return; }
	switch (real_id) {
		case 0:
			ValidateHost(lvalue, utils);
			break;
		case 1:
			ValidatePort(lvalue, utils);
			break;
		case 2:
			ValidateLogDuration(lvalue, utils);
			break;
		case 3:
			ValidateMessagesSentPerSecond(lvalue, utils);
			break;
		case 4:
			ValidateMessagesReceivedPerSecond(lvalue, utils);
			break;
		case 5:
			ValidateBytesSentPerSecond(lvalue, utils);
			break;
		case 6:
			ValidateBytesReceivedPerSecond(lvalue, utils);
			break;
		case 7:
			ValidateLocalLogPosition(lvalue, utils);
			break;
		case 8:
			ValidateRemoteLogPosition(lvalue, utils);
			break;
		case 9:
			ValidateLastMessageSent(lvalue, utils);
			break;
		case 10:
			ValidateLastMessageReceived(lvalue, utils);
			break;
		case 11:
			ValidateConnecting(lvalue, utils);
			break;
		case 12:
			ValidateSyncing(lvalue, utils);
			break;
		case 13:
			ValidateConnected(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<Endpoint>::NotifyField(int id, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::NotifyField(id, cookie); return; }
	switch (real_id) {
		case 0:
			NotifyHost(cookie);
			break;
		case 1:
			NotifyPort(cookie);
			break;
		case 2:
			NotifyLogDuration(cookie);
			break;
		case 3:
			NotifyMessagesSentPerSecond(cookie);
			break;
		case 4:
			NotifyMessagesReceivedPerSecond(cookie);
			break;
		case 5:
			NotifyBytesSentPerSecond(cookie);
			break;
		case 6:
			NotifyBytesReceivedPerSecond(cookie);
			break;
		case 7:
			NotifyLocalLogPosition(cookie);
			break;
		case 8:
			NotifyRemoteLogPosition(cookie);
			break;
		case 9:
			NotifyLastMessageSent(cookie);
			break;
		case 10:
			NotifyLastMessageReceived(cookie);
			break;
		case 11:
			NotifyConnecting(cookie);
			break;
		case 12:
			NotifySyncing(cookie);
			break;
		case 13:
			NotifyConnected(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<Endpoint>::NavigateField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::NavigateField(id); }
	throw std::runtime_error("Invalid field ID.");
}

String ObjectImpl<Endpoint>::GetHost() const
{
	return m_Host;
}

String ObjectImpl<Endpoint>::GetPort() const
{
	return m_Port;
}

double ObjectImpl<Endpoint>::GetLogDuration() const
{
	return m_LogDuration;
}

Timestamp ObjectImpl<Endpoint>::GetLocalLogPosition() const
{
	return m_LocalLogPosition;
}

Timestamp ObjectImpl<Endpoint>::GetRemoteLogPosition() const
{
	return m_RemoteLogPosition;
}

Timestamp ObjectImpl<Endpoint>::GetLastMessageSent() const
{
	return m_LastMessageSent;
}

Timestamp ObjectImpl<Endpoint>::GetLastMessageReceived() const
{
	return m_LastMessageReceived;
}

bool ObjectImpl<Endpoint>::GetConnecting() const
{
	return m_Connecting;
}

bool ObjectImpl<Endpoint>::GetSyncing() const
{
	return m_Syncing;
}

void ObjectImpl<Endpoint>::SetHost(const String& value, bool suppress_events, const Value& cookie)
{
	m_Host = value;
	if (!suppress_events)
		NotifyHost(cookie);
}

void ObjectImpl<Endpoint>::SetPort(const String& value, bool suppress_events, const Value& cookie)
{
	m_Port = value;
	if (!suppress_events)
		NotifyPort(cookie);
}

void ObjectImpl<Endpoint>::SetLogDuration(double value, bool suppress_events, const Value& cookie)
{
	m_LogDuration = value;
	if (!suppress_events)
		NotifyLogDuration(cookie);
}

void ObjectImpl<Endpoint>::SetMessagesSentPerSecond(double value, bool suppress_events, const Value& cookie)
{


	if (!suppress_events)
		NotifyMessagesSentPerSecond(cookie);
}

void ObjectImpl<Endpoint>::SetMessagesReceivedPerSecond(double value, bool suppress_events, const Value& cookie)
{


	if (!suppress_events)
		NotifyMessagesReceivedPerSecond(cookie);
}

void ObjectImpl<Endpoint>::SetBytesSentPerSecond(double value, bool suppress_events, const Value& cookie)
{


	if (!suppress_events)
		NotifyBytesSentPerSecond(cookie);
}

void ObjectImpl<Endpoint>::SetBytesReceivedPerSecond(double value, bool suppress_events, const Value& cookie)
{


	if (!suppress_events)
		NotifyBytesReceivedPerSecond(cookie);
}

void ObjectImpl<Endpoint>::SetLocalLogPosition(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_LocalLogPosition = value;
	if (!suppress_events)
		NotifyLocalLogPosition(cookie);
}

void ObjectImpl<Endpoint>::SetRemoteLogPosition(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_RemoteLogPosition = value;
	if (!suppress_events)
		NotifyRemoteLogPosition(cookie);
}

void ObjectImpl<Endpoint>::SetLastMessageSent(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_LastMessageSent = value;
	if (!suppress_events)
		NotifyLastMessageSent(cookie);
}

void ObjectImpl<Endpoint>::SetLastMessageReceived(const Timestamp& value, bool suppress_events, const Value& cookie)
{
	m_LastMessageReceived = value;
	if (!suppress_events)
		NotifyLastMessageReceived(cookie);
}

void ObjectImpl<Endpoint>::SetConnecting(bool value, bool suppress_events, const Value& cookie)
{
	m_Connecting = value;
	if (!suppress_events)
		NotifyConnecting(cookie);
}

void ObjectImpl<Endpoint>::SetSyncing(bool value, bool suppress_events, const Value& cookie)
{
	m_Syncing = value;
	if (!suppress_events)
		NotifySyncing(cookie);
}

void ObjectImpl<Endpoint>::SetConnected(bool value, bool suppress_events, const Value& cookie)
{


	if (!suppress_events)
		NotifyConnected(cookie);
}

void ObjectImpl<Endpoint>::NotifyHost(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnHostChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyPort(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPortChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyLogDuration(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnLogDurationChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyMessagesSentPerSecond(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnMessagesSentPerSecondChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyMessagesReceivedPerSecond(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnMessagesReceivedPerSecondChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyBytesSentPerSecond(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnBytesSentPerSecondChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyBytesReceivedPerSecond(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnBytesReceivedPerSecondChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyLocalLogPosition(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnLocalLogPositionChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyRemoteLogPosition(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnRemoteLogPositionChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyLastMessageSent(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnLastMessageSentChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyLastMessageReceived(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnLastMessageReceivedChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyConnecting(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnConnectingChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifySyncing(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnSyncingChanged(static_cast<Endpoint *>(this), cookie);
}

void ObjectImpl<Endpoint>::NotifyConnected(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnConnectedChanged(static_cast<Endpoint *>(this), cookie);
}

String ObjectImpl<Endpoint>::GetDefaultHost() const
{
	return String();
}

String ObjectImpl<Endpoint>::GetDefaultPort() const
{
	 return "5665"; 
}

double ObjectImpl<Endpoint>::GetDefaultLogDuration() const
{
	 return 86400; 
}

double ObjectImpl<Endpoint>::GetDefaultMessagesSentPerSecond() const
{
	return double();
}

double ObjectImpl<Endpoint>::GetDefaultMessagesReceivedPerSecond() const
{
	return double();
}

double ObjectImpl<Endpoint>::GetDefaultBytesSentPerSecond() const
{
	return double();
}

double ObjectImpl<Endpoint>::GetDefaultBytesReceivedPerSecond() const
{
	return double();
}

Timestamp ObjectImpl<Endpoint>::GetDefaultLocalLogPosition() const
{
	return Timestamp();
}

Timestamp ObjectImpl<Endpoint>::GetDefaultRemoteLogPosition() const
{
	return Timestamp();
}

Timestamp ObjectImpl<Endpoint>::GetDefaultLastMessageSent() const
{
	return Timestamp();
}

Timestamp ObjectImpl<Endpoint>::GetDefaultLastMessageReceived() const
{
	return Timestamp();
}

bool ObjectImpl<Endpoint>::GetDefaultConnecting() const
{
	return bool();
}

bool ObjectImpl<Endpoint>::GetDefaultSyncing() const
{
	return bool();
}

bool ObjectImpl<Endpoint>::GetDefaultConnected() const
{
	return bool();
}


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnHostChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnPortChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnLogDurationChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnMessagesSentPerSecondChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnMessagesReceivedPerSecondChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnBytesSentPerSecondChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnBytesReceivedPerSecondChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnLocalLogPositionChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnRemoteLogPositionChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnLastMessageSentChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnLastMessageReceivedChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnConnectingChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnSyncingChanged;


boost::signals2::signal<void (const intrusive_ptr<Endpoint>&, const Value&)> ObjectImpl<Endpoint>::OnConnectedChanged;

void ObjectImpl<Endpoint>::ValidateBytesReceivedPerSecond(const Lazy<double>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateBytesReceivedPerSecond(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateBytesSentPerSecond(const Lazy<double>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateBytesSentPerSecond(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateConnected(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateConnected(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateConnecting(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateConnecting(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateHost(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateHost(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateLastMessageReceived(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateLastMessageReceived(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateLastMessageSent(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateLastMessageSent(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateLocalLogPosition(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateLocalLogPosition(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateLogDuration(const Lazy<double>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateLogDuration(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateMessagesReceivedPerSecond(const Lazy<double>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateMessagesReceivedPerSecond(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateMessagesSentPerSecond(const Lazy<double>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateMessagesSentPerSecond(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidatePort(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePort(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateRemoteLogPosition(const Lazy<Timestamp>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateRemoteLogPosition(lvalue, utils);
}

void ObjectImpl<Endpoint>::ValidateSyncing(const Lazy<bool>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateSyncing(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
